#include "BufferSplit.h"
/*
A full line is divided into two: the first part and the second part.
head[] is copied from the first part.
split points the place next to the last character in the first part.

The second part is located as near to the end of tail[] as possible.
count points the first character in the second part.

split corresponds to the cursor.
count means the number of non-characters.
*/

bool BufferSplit::addMargin(int s) {
    using namespace std;

    const int HEAD_LENGTH = strlen(head);
    const int SIZE_1 = HEAD_LENGTH + MARGIN_SIZE + 1;
    if(SIZE_1 > enough) {
        return false;
    }

    if(split < s) {
        do {
            head[split] = tail[count];
            tail[count] = 0;
            count += 1;
            split += 1;
            head[split] = '\0';
        } while(split < s);

    } else if(split > s) {
        while(split > s) {
            count -= 1;
            split -= 1;
            tail[count] = head[split];
            head[split] = '\0';
        }

    }

//    const int ENOUGH_1 = enough - 1;
    const int ENOUGH_2 = enough - 2;

    tail[ count - 2 ] = 0;
    for(int k = count - 1; k < ENOUGH_2; ++k) {
        tail[k] = tail[ k + 1 ];
    }

    tail[ENOUGH_2] = ' '; // MARGIN
//    tail[ENOUGH_1] = '\0';

    count -= 1;
    return true;
}

bool BufferSplit::squeezing(std::string & data, int s) {
    using namespace std;

    const int DATA_LENGTH = data.length();
    const int HEAD_LENGTH = strlen(head);
    const int SIZE_1 = DATA_LENGTH + HEAD_LENGTH + 1;
    if(SIZE_1 > enough) {
        return false;
    }

    if(split < s) {
        do {
            head[split] = tail[count];
            tail[count] = 0;
            count += 1;
            split += 1;
            head[split] = '\0';
        } while(split < s);

    } else if(split > s) {
        while(split > s) {
            count -= 1;
            split -= 1;
            tail[count] = head[split];
            head[split] = '\0';
        }

    }

    strcat(head, data.c_str() );
    split += DATA_LENGTH;
    return true;
}

BufferSplit::BufferSplit(std::string & data, std::string & f, int s): MARGIN(" ") {
    using namespace std;

    MARGIN_SIZE = MARGIN.length();
    fullLine = f;
    split = s;

    string h = fullLine.substr(0, split);
    string t = fullLine.substr(split);
    string u = data + fullLine;

    const int t_length = t.length();
    enough = HUGE_MEMORY_REQUIRED * u.length() + BUFFER_SIZE;
    const int SIZE_1 = enough - 1;

    full = new char[enough];
    head = new char[enough];
    tail = new char[enough];

    memset(tail, 0, enough);
    count = SIZE_1 - t_length; // count means the number of non-characters.
    char *p = & tail[count];
    strcpy(p, t.c_str() );

    const int DATA_LENGTH = data.length();
    h += data;
    split += DATA_LENGTH;
    strcpy(head, h.c_str() );
}

BufferSplit::BufferSplit(std::string & f, int s, int delete_characters, bool margin): MARGIN(" ") {
    using namespace std;

    MARGIN_SIZE = MARGIN.length();
    if(margin) {
        fullLine = f + MARGIN;
    } else {
        fullLine = f;
    }

    split = s;

    string h = fullLine.substr(0, split);
    int from = split + delete_characters;
    string t = fullLine.substr(from);

    const int t_length = t.length();
    enough = HUGE_MEMORY_REQUIRED * fullLine.length() + BUFFER_SIZE;
    const int SIZE_1 = enough - 1;

    full = new char[enough];
    head = new char[enough];
    tail = new char[enough];

    memset(tail, 0, enough);
    count = SIZE_1 - t_length; // count means the number of non-characters.
    char *p = & tail[count];
    strcpy(p, t.c_str() );

    strcpy(head, h.c_str() );
}

int BufferSplit::addData(std::string & data) {
    using namespace std;

    const int SIZE_1 = enough - 1;

// head[] is copied from the full line.
    char *p = & tail[count];
    strcat(head, p);
    int split = strlen(head);

// data is located as near to the end of tail[] as possible.
    memset(tail, 0, enough);
    count = SIZE_1 - data.length(); // count means the number of non-characters.
    p = & tail[count];
    strcpy(p, data.c_str() );

    return split;
}

int BufferSplit::clearPart() {
    using namespace std;

    const int SIZE_1 = enough - 1;
    int split = strlen(head);

// Clearing the second part
    memset(tail, 0, enough);
    count = SIZE_1;

    return split;
}

std::string BufferSplit::unified() {
    using namespace std;

    char *p = & tail[count];
    strcpy(full, head);
    strcat(full, p);

    const string FULL_LINE(full);
    fullLine = FULL_LINE;
    return fullLine;
}

void BufferSplit::excise(int s, int d) {
    if(split < s) {
        do {
            head[split] = tail[count];
            tail[count] = 0;
            count += 1;
            split += 1;
            head[split] = '\0';
        } while(split < s);

    } else if(split > s) {
        while(split > s) {
            count -= 1;
            split -= 1;
            tail[count] = head[split];
            head[split] = '\0';
        }

    }

    for(int i = 0; i < d; ++i) {
        tail[count] = 0;
        count += 1;
    }

}

void BufferSplit::trimming(int s) {
    if(split < s) {
        do {
            head[split] = tail[count];
            tail[count] = 0;
            count += 1;
            split += 1;
            head[split] = '\0';
        } while(split < s);

    } else if(split > s) {
        while(split > s) {
            count -= 1;
            split -= 1;
            tail[count] = head[split];
            head[split] = '\0';
        }

    }

    const int SIZE_1 = enough - 1;
//    int k = enough - 1;
//    tail[k] = '\0';

    for(int k = enough - 2; k > count; --k) {
        tail[k] = tail[ k - 1 ];
    }

    tail[count] = 0;
    if(count < SIZE_1) {
        count += 1;
    }

}
