/*
Editor Version 1.11
Showing the Editor window of Zoko
Copyright 2017 Sogaya

Borland C++Builder 5

*/

#include <vcl.h>
#include "editorMain.h"
#pragma hdrstop
USERES("editor.res");
USEFORM("editorMain.cpp", Form1);
USEUNIT("OS-specific.cpp");
USEUNIT("Dividing.cpp");
USEUNIT("TimeName.cpp");
USEUNIT("BufferSplit.cpp");
USEFORM("editorJump.cpp", OKBottomDlg);
//---------------------------------------------------------------------------
EDITOR_MEMO editorMemo;
EDITOR_MEMO *em = &editorMemo;

namespace {
    using namespace std;

    const bool WITH_COMMENTS = true;
    const bool WITHOUT_COMMENTS = false;
    const int STANDARD_LINE_LENGTH = 32; // "10 07 12 17 23 28 34 03 15 -A -1"
    const string CLOSE_BRACKET(" )");
    const string LINE_1("18 21 29 09 03 27 23 35 37 -F -222");
    const string NO_REMOVING = "-r-";
    const string QUIT = "-q";
    const string VERSION_COMMENTS = "-VC";
    const string VERSION_GUIDE = "-V";

    string error("");
}

bool readFilesBeforeCreatingWindows(std::string & commandLine) {
    using namespace std;

    bool quit = false;
    bool removeFirst = true;
    char buffer[BUFFER_SIZE];
    Dividing d(commandLine);
    const int FIELD_COUNT = d.fieldCount();
    int q;
    string *firstLine = new string(LINE_1);
    string argument;
    string fullPathName;
    string manage_text(MANAGE_TEXT);
    string who_text(WHO_TEXT);

// C:\zokoVCL\manage.txt for example
    string firstPart = currentDirectory(); // OS-specific.h and OS-specific.cpp
    NAMES a = driveDirectoryBase(firstPart);
    string SEPARATOR = a.separator;
    string root = a.name[1];
    if (firstPart == root) { // C:\ for example
        fullPathName = root + manage_text;
    } else { // C:\zokoVCL for example
        fullPathName = firstPart + SEPARATOR + manage_text;
    }

    ifstream file( fullPathName.c_str() );
    if ( file.fail() ) {
        error = "ifstream file( " + fullPathName + CLOSE_BRACKET;
    } else {
        file.close();
    }

// Acquiring firstLine from WHO_TEXT
    ifstream in(WHO_TEXT);
    if( in.fail() ) {
        error = "e1: ifstream in( " + who_text + CLOSE_BRACKET;
        delete firstLine;
        return false;
    }

    if( in.getline(buffer, BUFFER_SIZE) ) {
        delete firstLine;
        firstLine = new string(buffer);
    } else {
        error = "e3: ifstream in( " + who_text + CLOSE_BRACKET;
        delete firstLine;
        return false;
    }

    in.close();

// parameters
    for(int i = 1; i < FIELD_COUNT; ++i) {
        argument = d.a(i);
        if( argument.length() == 0 ) {
            break;
        }

        if(NO_REMOVING == argument) {
            removeFirst = false;
        } else if(QUIT == argument) {
            quit = true;
        } else if(VERSION_COMMENTS == argument) {
            guide(WITH_COMMENTS);
            delete firstLine;
            return false;

        } else if(VERSION_GUIDE == argument) {
            guide(WITHOUT_COMMENTS);
            delete firstLine;
            return false;

        } else {
            delete firstLine;
            firstLine = new string(argument);
        }

    }

    if(removeFirst) {
        removeTemporaryFiles();
    }

    string line_1 = *firstLine;
    if( line_1.length() < STANDARD_LINE_LENGTH ) {
        error = "e4: \"" + line_1 + "\".length() < STANDARD_LINE_LENGTH ";
        delete firstLine;
        return false;
    }

    if(quit) {
        return false;
    }

    em->fullPathName = fullPathName;
    em->line_1 = line_1;
    em->versionNumber = VERSION;
    return true;
}

NAMES driveDirectoryBase(std::string & s) {
    using namespace std;

    const string EMPTY = "";
    const string SLASH = "/";
    const string YEN = "\\";
    int firstTrace, trace, w;
    int s_length = s.length();
    string c;

    NAMES part, undivided;
    part.name[0] = EMPTY; part.name[1] = EMPTY; part.name[2] = EMPTY; part.name[3] = EMPTY;
    undivided.name[0] = EMPTY; undivided.name[1] = EMPTY; undivided.name[2] = EMPTY; undivided.name[3] = EMPTY;

    part.name[0] = s; // s.length > 0; in other words, part.name[0].length > undivided.name[0].length

// s is divided into three parts: drive name, directory name, and base name.
    int k = 0;
    for( ; k < s_length; ++k) {
        c = s.substr(k, 1);
        if(c == YEN) {
            part.separator = YEN;
            break;
        } else if(c == SLASH) {
            part.separator = SLASH;
            break;
        }

    }

    if(k == s_length) { // Neither YEN nor SLASH has been found.
        return undivided;
    } else {
        w = k + 1;
        part.name[1] = s.substr(0, w); // C:\ for example
        trace = ++k;
        firstTrace = trace;
    }

    for( ; k < s_length; ++k) {
        c = s.substr(k, 1);
        if(c == part.separator) {
            trace = ++k;
        }

    }

    w = trace - firstTrace;
    part.name[2] = s.substr(firstTrace, w);
    part.name[3] = s.substr(trace); // manage.txt for example
    return part;
}

void createErrorText(std::string & messages) {
    using namespace std;

    ofstream file(ERROR_TEXT);
    file << messages << endl;
}

void guide(bool comments) {
    using namespace std;

    if(comments) {
        ofstream file(VERSION_TEXT);
        file << "editor Version " << VERSION << " \n" << endl 
            << "editor          # Show the Editor window. " << endl 
            << "editor -q       # Quit before showing the Editor window. " << endl 
            << "editor -r-      # Do not remove temporary files. " << endl 
            << "editor -V       # version " << endl 
            << "editor -VC      # version with comments " << endl;
    } else {
        ofstream file(VERSION_TEXT);
        file << "editor Version " << VERSION << " \n" << endl 
            << "editor          " << endl 
            << "editor -q       " << endl 
            << "editor -r-      " << endl 
            << "editor -V       " << endl 
            << "editor -VC      " << endl;
    }

}

bool removeTemporaryFiles() {
    using namespace std;

    bool match;
    char c[FILE_NAME_SIZE];
    ifstream in(REMOVE_ERROR_TEXT);
    if( in.fail() ) {
        return false;
    }

    while( in.getline(c, FILE_NAME_SIZE) ) {
        if( strcmp(c, SEE_TEXT) ) { // Do not remove SEE_TEXT.
            if( strcmp(c, TABULATE_TEXT) ) { // Do not remove TABULATE_TEXT.
                if( strcmp(c, TRUE_TEXT) ) { // Do not remove TRUE_TEXT.
                    if( strcmp(c, WHO_TEXT) ) { // Do not remove WHO_TEXT.
                        remove(c);
                    }

                }

            }

        }

    }

    return true;
}

WINAPI WinMain(HINSTANCE, HINSTANCE, LPSTR, int)
{
    std::string command( ::GetCommandLine() );
    if( readFilesBeforeCreatingWindows(command) ) {
        ;
    } else {
        createErrorText(error);
        return 0;
    }

    try
    {
         Application->Initialize();
         Application->Title = "Editor";
         Application->HelpFile = "C:\\zokoVCL\\ZOKO.HLP";
         Application->CreateForm(__classid(TForm1), &Form1);
         Application->CreateForm(__classid(TOKBottomDlg), &OKBottomDlg);
         Application->Run();
    }
    catch (Exception &exception)
    {
         Application->ShowException(&exception);
    }
    return 0;
}
//---------------------------------------------------------------------------
