/*
Editor Version 1.11
Showing the Editor window of Zoko
Copyright 2017 Sogaya

Borland C++Builder 5

*/

#include <vcl.h>
#pragma hdrstop

#include "editorJump.h"
#include "editorMain.h"

#pragma package(smart_init)
#pragma resource "*.dfm"

extern EDITOR_MEMO *em;
extern TOKBottomDlg *OKBottomDlg;
TEXT_X text_x;
TForm1 *Form1;

bool TForm1::openUndo() {
    using namespace std;

    const bool SERIAL_NUMBER = true;
    TimeName undo_bin(UND, BINARY_FILE);

    timeNameText = undo_bin.temporaryName(SERIAL_NUMBER);

    if(! undoClosed) {
        putUndo.close();
    }

    putUndo.open( timeNameText.c_str(), ios::out | ios::binary );
    if( putUndo.fail() ) {
        return false;
    }

    undoClosed = false;
    recordNumber = 1;

    if(editMode) {
        delete bs;
        bs = NULL;
        editMode = false;
    }

    return true;
}

bool TForm1::readManageText() {
    using namespace std;

    char c[SMALL_BUFFER_SIZE];
    const string manage_text(MANAGE_TEXT);
    ifstream in(MANAGE_TEXT);
    if( in.fail() ) {
        error = "e5: ifstream in( " + manage_text + CLOSE_BRACKET;
        return false;
    }

    statusBarText = manage_text;
    for(int line = 1; line <= MANAGE_TEXT_LINE_COUNT; line += 1) {
        if( in.getline(c, SMALL_BUFFER_SIZE) ) {
            if(line == EDITOR_LINE) {
                if(! strcmp(c, "0") ) {
                    Close();
                }

                break;
            }

        } else {
            break;
        }

    }

    in.close();
    return true;
}

bool TForm1::readUndoBin() {
    using namespace std;

    if(recordNumber == 1) {
        return false;
    }

    if(! undoClosed) {
        putUndo.close();
        undoClosed = true;
    }

    getUndo.open( timeNameText.c_str(), ios::in | ios::binary );
    if( getUndo.fail() ) {
        error = "e3: getUndo.open( " + timeNameText + " , ios::in | ios::binary" + CLOSE_BRACKET;
        return false;
    }

    recordNumber -= 1;

    streampos begin = RECORD_SIZE * ( recordNumber - 1 );
    getUndo.seekg(begin);
    getUndo.read(record, RECORD_SIZE);
    getUndo.close();

    int reverse_command = record[0];
    cursor = record[1];
    ud.previous_aim = record[2];
    ud.previous_x = record[3];
    ud.previous_block_x = record[4];
    ud.size_bytes = record[5];

// little-endian architecture
    r.number_bytes[0] = record[ PARAMETER_COUNT - 4 ];
    r.number_bytes[1] = record[ PARAMETER_COUNT - 3 ];
    r.number_bytes[2] = record[ PARAMETER_COUNT - 2 ];
    r.number_bytes[3] = record[ PARAMETER_COUNT - 1 ];

    char *non_parameter = & record[PARAMETER_COUNT];
    strcpy(ud.data, non_parameter);

    switch(reverse_command) {
    case UNDO_BS:
        undoBS();
        break;
    case UNDO_CLEAR:
        undoClear();
        break;
    case UNDO_CLIP:
        undoClip();
        break;
    case UNDO_CUT:
        undoCut();
        break;
    case UNDO_CUT_ADD_MARGIN:
        undoCutAddMargin();
        break;
    case UNDO_DELETE:
        undoDelete();
        break;
    case UNDO_GO:
        undoGo();
        break;
    case UNDO_INSERT:
        undo_insert();
        break;
    case UNDO_MARGIN:
        undoMargin();
        break;
    case UNDO_SEVER:
        undoSever();
        break;
    case UNDO_SHRINK:
        undoShrink();
        break;
    case UNDO_START:
        undoStart();
        break;
    case UNDO_SWELL:
        undoSwell();
        break;
    }

    return true;
}

bool TForm1::undoThat() {
    bool no_error = true;
    bool read_to_undo = false;

    if(recordNumber == 1) {
        return no_error;
    }

    if(goMode) {
        read_to_undo = true;
    } else {
        if(blockMode) {
            read_to_undo = true;
        }

    }

    if(read_to_undo) {
        if( readUndoBin() ) {
            no_error = true;
        } else {
            no_error = false;
        }

    }

    return no_error;
}

bool TForm1::saveLine1() {
    using namespace std;

    char c[BUFFER_SIZE];
    const string NEWLINE("\x0A");
    const string who_text(WHO_TEXT);
    string s;
    stringstream os;
    ifstream in(WHO_TEXT);
    if( in.fail() ) {
        error = "e8: ifstream in( " + who_text + CLOSE_BRACKET;
        return false;
    }

    s = trimmed(statusBarText);
    if( in.getline(c, BUFFER_SIZE) ) {
        os << s << NEWLINE;
    } else {
        error = "e9(Line-1): if( in.getline(c, BUFFER_SIZE)" + CLOSE_BRACKET;
        return false;
    }

    for(int line = 2; line <= CSV_LINE_COUNT; line += 1) {
        if( in.getline(c, BUFFER_SIZE) ) {
            os << c << NEWLINE;
        } else {
            break;
        }

    }

    in.close();

    s = os.str();
    ofstream whoText(WHO_TEXT);
    whoText << s;
    return true;
}

int modified_file(const char *file_name) {
    AnsiString FILE_NAME(file_name);
    return FileAge(FILE_NAME);
}

std::string TForm1::clearBlock() {
    using namespace std;

    string s = copied;
    if(blockMode) {
        blockMode = false;

// Ending the Block mode
        string gap_added = GAP + statusBarText;
        int GAP_ADDED_LENGTH = gap_added.length();

        if(cursor < 1) {
            return s;
        } else if(cursor >= GAP_ADDED_LENGTH) {
            return s;
        }

        for(int i = anchor; i <= cursor; ++i) {
            whiteSquare(image[i]);
            blackCharacter(image[i], gap_added, i);
        }

// clipboard
        char buffer[SMALL_BUFFER_SIZE];
        int count = cursor + 1 - anchor;
        s = gap_added.substr(anchor, count);
        strcpy(buffer, s.c_str() );
        Clipboard()->SetTextBuf(buffer);

// undo data
        ud.previous_block_x = anchor;
        ud.previous_aim = aim;
        ud.reverse_command = UNDO_CLEAR;
        ud.cell_x = cursor;
        ud.previous_x = cursor;
        memset(ud.data, 0, IMAGE_COUNT);
        ud.size_bytes = 0;
        addUndoData();
    }

    return s;
}

std::string TForm1::copyHandler() {
    char buffer[SMALL_BUFFER_SIZE];
    strcpy(buffer, statusBarText.c_str() );
    Clipboard()->SetTextBuf(buffer);
    return statusBarText;
}

std::string TForm1::cutAddMargin() {
    using namespace std;

    string s = copied;
    if(blockMode) {
        blockMode = false;

// Ending the Block mode
        string old_gap_added = GAP + statusBarText;
        int OLD_GAP_ADDED_LENGTH = old_gap_added.length();

        for(int i = anchor; i <= cursor; ++i) {
            whiteSquare(image[i]);
        }

// clipboard
        char buffer[SMALL_BUFFER_SIZE];
        const int DATA_LENGTH = cursor + 1 - anchor;
        s = old_gap_added.substr(anchor, DATA_LENGTH);
        strcpy(buffer, s.c_str() );
        Clipboard()->SetTextBuf(buffer);

// Deleting characters
        const bool ADD_MARGIN = true;
        ud.previous_aim = aim;
        aim = GO;
        goMode = true;

        ud.previous_x = cursor;
        cursor = anchor;

        string gap_added = startEditing(old_gap_added, DATA_LENGTH, ADD_MARGIN);
        int GAP_ADDED_LENGTH = gap_added.length();
        statusBarText = gap_added.substr(GAP_SIZE);

// Moving the cursor
        cursor = anchor - 1;
        redSquare(image[cursor]);
        whitenBlacken(old_gap_added, gap_added, OLD_GAP_ADDED_LENGTH, GAP_ADDED_LENGTH);

// undo data
        ud.cell_x = cursor;
        ud.previous_block_x = anchor;
        ud.reverse_command = UNDO_CUT_ADD_MARGIN;
        strcpy(ud.data, buffer);
        ud.size_bytes = DATA_LENGTH;
        addUndoData();
    }

    return s;
}

std::string TForm1::cutHandler() {
    using namespace std;

    string s = copied;
    if(blockMode) {
        string old_gap_added = GAP + statusBarText;
        int OLD_GAP_ADDED_LENGTH = old_gap_added.length();

        if(cursor < anchor) {
            return s;
        } else if(cursor >= OLD_GAP_ADDED_LENGTH) {
            return s;
        } else if( cursor == OLD_GAP_ADDED_LENGTH - 1 ) {
            if(anchor == 1) {
                return whiteHandler();
            } else {
                return cutAddMargin();
            }

        }

        blockMode = false;

// Ending the Block mode
        for(int i = anchor; i <= cursor; ++i) {
            whiteSquare(image[i]);
        }

// clipboard
        char buffer[SMALL_BUFFER_SIZE];
        int count = cursor + 1 - anchor;
        s = old_gap_added.substr(anchor, count);
        strcpy(buffer, s.c_str() );
        Clipboard()->SetTextBuf(buffer);

// Deleting characters
        const bool NO_MARGIN = false;
        ud.previous_aim = aim;
        aim = GO;
        goMode = true;

        ud.previous_x = cursor;
        cursor = anchor;

        string gap_added = startEditing(old_gap_added, count, NO_MARGIN);
        int GAP_ADDED_LENGTH = gap_added.length();

        statusBarText = gap_added.substr(GAP_SIZE);
        redSquare(image[cursor]);
        whitenBlacken(old_gap_added, gap_added, OLD_GAP_ADDED_LENGTH, GAP_ADDED_LENGTH);

// undo data
        ud.previous_block_x = anchor;
        ud.reverse_command = UNDO_CUT;
        ud.cell_x = cursor;
        strcpy(ud.data, buffer);
        ud.size_bytes = count;
        addUndoData();
    }

    return s;
}

std::string TForm1::deleted(std::string & old_gap_added) {
    const bool NO_MARGIN = false;
    const int DELETE_ONE_CHARACTER = 1;
    std::string s = old_gap_added;

    if(editMode) {
        bs->excise(cursor, DELETE_ONE_CHARACTER);
        s = bs->unified();
    } else {
        s = startEditing(s, DELETE_ONE_CHARACTER, NO_MARGIN);
    }

    return s;
}

std::string TForm1::inserted(std::string & data, std::string & old_gap_added) {
    std::string s = old_gap_added;

    if(editMode) {
        if( bs->squeezing(data, cursor) ) {
            s = bs->unified();
        } else {
            error = "e4: bs->squeezing(data, cursor" + CLOSE_BRACKET;
            s = EMPTY;
        }

    } else {
        s = startEditing(data, s);
    }

    return s;
}

std::string TForm1::startEditing(std::string & data, std::string & s) {
    std::string fullLine;

    if(editMode) {
        ;
    } else {
        editMode = true;
        bs = new BufferSplit(data, s, cursor);
        fullLine = bs->unified();
    }

    return fullLine;
}

std::string TForm1::startEditing(std::string & s, int count, bool margin) {
    std::string fullLine;

    if(editMode) {
        ;
    } else {
        editMode = true;
        bs = new BufferSplit(s, cursor, count, margin);
        fullLine = bs->unified();
    }

    return fullLine;
}

std::string TForm1::trimmed(std::string & line) {
    using namespace std;

    string c;
    string s = line + GAP;
    int size = s.length();

// Only a GAP shall appear at the end of Line-1.  
    for(int i = size - 1 ; i >= 0; --i) {
        c = s.substr(i, 1);
        if(GAP == c) {
            continue;
        } else {
            ++i;
            s = s.substr(0, i + GAP_SIZE );
            break;
        }

    }

    return s;
}
void TForm1::acquireDataFromClipBoard() {
    using namespace std;

    const int IMAGE_COUNT_1 = IMAGE_COUNT - 1;
    int SIZE = BUFFER_SIZE;
    char *buffer = new char[ SIZE + 1 ];
    Clipboard()->GetTextBuf(buffer, SIZE);

    const string COPIED(buffer);
    if( COPIED.length() > IMAGE_COUNT_1 ) {
        copied = COPIED.substr(0, IMAGE_COUNT_1);
    } else {
        copied = COPIED;
    }

    insertHandler(copied);
    delete buffer;
}

void TForm1::addUndoData() {
    record[0] = (char)ud.reverse_command;
    record[1] = (char)ud.cell_x;
    record[2] = (char)ud.previous_aim;
    record[3] = (char)ud.previous_x;
    record[4] = (char)ud.previous_block_x;
    record[5] = (char)ud.size_bytes;

    if(undoClosed) {
        openUndo();
    }

    char *p = & record[PARAMETER_COUNT];
    ud.data[MAXIMUM_DATA_SIZE] = '\0';
    memcpy(p, ud.data, MAXIMUM_DATA_SIZE + 1 );

    std::streampos begin = RECORD_SIZE * ( recordNumber - 1 );
    putUndo.seekp(begin);
    putUndo.write(record, RECORD_SIZE);

// Setting the next record number
    recordNumber += 1;

// little-endian architecture
    r.record_number = recordNumber;
    record[ PARAMETER_COUNT - 4 ] = r.number_bytes[0];
    record[ PARAMETER_COUNT - 3 ] = r.number_bytes[1];
    record[ PARAMETER_COUNT - 2 ] = r.number_bytes[2];
    record[ PARAMETER_COUNT - 1 ] = r.number_bytes[3];
}

void TForm1::blackCharacter(TImage *image, std::string gap_added, int cell_x) {
    using namespace std;

// Writing a black character
    TCanvas *p = image->Canvas;
    p->Font->Name = "Arial";
    p->Font->Size = 10;
    p->Font->Color = clBlack;
    SetBkMode(p->Handle, TRANSPARENT);

    string c = gap_added.substr(cell_x, 1);
    p->TextOut(text_x[c], TEXT_Y, c.c_str() );
}

void TForm1::BShandler() {
    if(goMode) {
        using namespace std;

        if(cursor == 1) {
            return;
        }

        const int TRACE = cursor;
        string old_gap_added = GAP + statusBarText;
        int OLD_GAP_ADDED_LENGTH = old_gap_added.length();

        if(TRACE < 1) {
            homeHandler();
            return;
        } else if(TRACE >= OLD_GAP_ADDED_LENGTH) {
            endHandler();
            return;
        }

        cursor -= 1;
        if(cursor < 1) {
            homeHandler();
            return;
        }

        string data = old_gap_added.substr(cursor, 1);
        string gap_added = deleted(old_gap_added);
        int GAP_ADDED_LENGTH = gap_added.length();
        statusBarText = gap_added.substr(GAP_SIZE);

        if( GAP_ADDED_LENGTH == GAP_SIZE + 1 ) { // gap_added includes a gap and a character.
            endEditing();
            goMode = false;

// Ending the Go mode
            whiteSquare(image[TRACE]);

            for(int i = cursor; i <= TRACE; ++i) {
                whiteCharacter(image[i], old_gap_added, i);
            }

            blackCharacter(image[cursor], gap_added, cursor);
            return;
        }

        whiteSquare(image[TRACE]);
        redSquare(image[cursor]);
        whitenBlacken(old_gap_added, gap_added, OLD_GAP_ADDED_LENGTH, GAP_ADDED_LENGTH);

// undo data
        ud.cell_x = cursor;
        ud.previous_aim = aim;
        ud.previous_block_x = anchor;
        ud.previous_x = TRACE;
        ud.reverse_command = UNDO_BS;
        strcpy(ud.data, data.c_str() );
        ud.size_bytes = 1; // data.length();
        addUndoData();
    }

}

void TForm1::clipping() {
    if(goMode) {
        using namespace std;

        string data;
        string gs = GAP + statusBarText;

        string old_gap_added = gs.substr(0, GAP_IMAGE_COUNT);
        data = gs.substr(GAP_IMAGE_COUNT);

// clipboard
        char buffer[LARGE_BUFFER_SIZE];
        strcpy(buffer, data.c_str() );
        Clipboard()->SetTextBuf(buffer);

// Temporarily adding margin
        const bool ADD_MARGIN = true;
        const int DATA_LENGTH = data.length();
        const int DELETE_NOTHING = 0;
        const int TRACE = cursor;

        cursor = GAP_IMAGE_COUNT; // cursor = IMAGE_COUNT + 1;
        if(editMode) {
            bs->addMargin(cursor);
        } else {
            editMode = true;
            bs = new BufferSplit(old_gap_added, cursor, DELETE_NOTHING, ADD_MARGIN);
        }

// Deleting
        bs->excise(cursor, DATA_LENGTH);
        bs->trimming(cursor);
        string gap_added = bs->unified();

        statusBarText = gap_added.substr(GAP_SIZE);

        cursor = IMAGE_COUNT;
        whiteSquare(image[TRACE]);
        redSquare(image[cursor]);

// Writing black characters
        blackCharacter(image[TRACE], gap_added, TRACE);
        blackCharacter(image[cursor], gap_added, cursor);

// undo data
        ud.cell_x = cursor;
        ud.previous_aim = aim;
        ud.previous_block_x = anchor;
        ud.previous_x = TRACE;
        ud.reverse_command = UNDO_CLIP;
        strcpy(ud.data, data.c_str() );
        ud.size_bytes = DATA_LENGTH;
        addUndoData();
    }

}

void TForm1::cursorLine() {
    using namespace std;

    string c;
    string gap_added = GAP + statusBarText;
    int gap_added_length = gap_added.length();
    TCanvas *p;

// Whitening to erase characters
    for(int i = 1; i <= IMAGE_COUNT; ++i) {
        p = image[i]->Canvas;
        p->Font->Name = "Arial";
        p->Font->Size = 10;

        if(i == cursor) {
            if(goMode) {
                p->Font->Color = clRed;
            } else {
                p->Font->Color = clWhite;
            }

        } else {
            p->Font->Color = clWhite;
        }

        SetBkMode(p->Handle, TRANSPARENT);

        if(i < gap_added_length) {
            c = gap_added.substr(i, 1);
            p->TextOut(text_x[c], TEXT_Y, c.c_str() );
        } else {
            p->TextOut(text_x[GAP], TEXT_Y, GAP.c_str() );
        }

    }

// Writing black characters
    gap_added_length = gap_added.length();

    for(int i = 1; i <= IMAGE_COUNT; ++i) {
        p = image[i]->Canvas;

// white background
        p->Brush->Color = clWhite;
        p->FillRect( Rect(0, 0, IMAGE_WIDTH, IMAGE_HEIGHT) );

// black character
        p->Font->Name = "Arial";
        p->Font->Size = 10;
        p->Font->Color = clBlack;
        SetBkMode(p->Handle, TRANSPARENT);

        if(i < gap_added_length) {
            c = gap_added.substr(i, 1);
            p->TextOut(text_x[c], TEXT_Y, c.c_str() );
        } else {
            p->TextOut(text_x[GAP], TEXT_Y, GAP.c_str() );
        }
    }

    return;
}

void TForm1::deleteHandler() {
    if(goMode) {
        using namespace std;

        const int TRACE = cursor;
        string old_gap_added = GAP + statusBarText;
        int OLD_GAP_ADDED_LENGTH = old_gap_added.length();
        TCanvas *p;

        if(TRACE < 1) {
            homeHandler();
            return;
        } else if(TRACE >= OLD_GAP_ADDED_LENGTH) {
            endHandler();
            return;
        }

        string data = old_gap_added.substr(cursor, 1);
        string gap_added = deleted(old_gap_added);
        int GAP_ADDED_LENGTH = gap_added.length();

        if(GAP_ADDED_LENGTH == GAP_SIZE) { // gap_added includes only a gap because the status bar text is empty.
            endEditing();
            goMode = false;

// Ending the Go mode
            whiteSquare(image[1]);

            p = image[cursor]->Canvas;
            p->Font->Name = "Arial";
            p->Font->Size = 10;
            p->Font->Color = clBlack;
            SetBkMode(p->Handle, TRANSPARENT);

            p->TextOut(text_x[GAP], TEXT_Y, GAP.c_str() );
            return;
        }

        statusBarText = gap_added.substr(GAP_SIZE);

        if(cursor == GAP_ADDED_LENGTH) {
            whiteSquare(image[cursor]);
            cursor = GAP_ADDED_LENGTH - 1;
        }

        redSquare(image[cursor]);
        whitenBlacken(old_gap_added, gap_added, OLD_GAP_ADDED_LENGTH, GAP_ADDED_LENGTH);

// undo data
        ud.cell_x = cursor;
        ud.previous_aim = aim;
        ud.previous_block_x = anchor;
        ud.previous_x = TRACE;
        ud.reverse_command = UNDO_DELETE;
        strcpy(ud.data, data.c_str() );
        ud.size_bytes = 1; // data.length();
        addUndoData();
    }

}

void TForm1::endEditing() {
    if(editMode) {
        delete bs;
        bs = NULL;
        editMode = false;

// Before undo
        if(! undoClosed) {
            putUndo.close();
            undoClosed = true;
            openUndo();
        }

    }

}

void TForm1::endHandler() {
    if(goMode) {
        using namespace std;

        int trace = cursor;
        string gap_added = GAP + statusBarText;
        const int GAP_ADDED_LENGTH = gap_added.length();

        if(trace < 1) {
            homeHandler();
            return;
        } else if(trace >= GAP_ADDED_LENGTH) {
            trace = GAP_ADDED_LENGTH - 1;
        }

        cursor = GAP_ADDED_LENGTH - 1;

        whiteSquare(image[trace]);
        redSquare(image[cursor]);

// Writing black characters
        blackCharacter(image[trace], gap_added, trace);
        blackCharacter(image[cursor], gap_added, cursor);

// undo data
        ud.reverse_command = UNDO_GO;
        ud.cell_x = cursor;
        ud.previous_aim = aim;
        ud.previous_x = trace;
        ud.previous_block_x = NO_BLOCK;
        memset(ud.data, 0, IMAGE_COUNT);
        ud.size_bytes = 0;
        addUndoData();
    }

}

void TForm1::hideHandler() {
    if(goMode) {
        using namespace std;

        goMode = false;

// Ending the Go mode
        string gap_added = GAP + statusBarText;
        const int GAP_ADDED_LENGTH = gap_added.length();

        if(cursor < 1) {
            homeHandler();
            return;
        } else if(cursor >= GAP_ADDED_LENGTH) {
            endHandler();
            return;
        }

        whiteSquare(image[cursor]);
        blackCharacter(image[cursor], gap_added, cursor);
    }

}

void TForm1::homeHandler() {
    if(goMode) {
        using namespace std;

        int trace = cursor;
        string gap_added = GAP + statusBarText;
        const int GAP_ADDED_LENGTH = gap_added.length();

        if(trace < 1) {
            trace = 1;
        } else if(trace >= GAP_ADDED_LENGTH) {
            endHandler();
            return;
        }

        cursor = 1;

        whiteSquare(image[trace]);
        redSquare(image[cursor]);

// Writing black characters
        blackCharacter(image[trace], gap_added, trace);
        blackCharacter(image[cursor], gap_added, cursor);

// undo data
        ud.reverse_command = UNDO_GO;
        ud.cell_x = cursor;
        ud.previous_aim = aim;
        ud.previous_x = trace;
        ud.previous_block_x = NO_BLOCK;
        memset(ud.data, 0, IMAGE_COUNT);
        ud.size_bytes = 0;
        addUndoData();
    }

}

void TForm1::insertHandler(std::string & data) {
    if(goMode) {
        using namespace std;

        if(cursor == IMAGE_COUNT) {
            return;
        }

        const int DATA_LENGTH = data.length();
        const int CURSOR_DATA_LENGTH = cursor + DATA_LENGTH;
        const int TRACE = cursor;
        string c;
        string old_gap_added = GAP + statusBarText;
        const int OLD_GAP_ADDED_LENGTH = old_gap_added.length();
        TCanvas *p;

        if(TRACE < 1) {
            homeHandler();
            return;
        } else if(TRACE >= OLD_GAP_ADDED_LENGTH) {
            endHandler();
            return;
        }

        if(CURSOR_DATA_LENGTH > IMAGE_COUNT) {
            endEditing();
            repeat_inserting(data);
            return;
        }

        string gap_added = inserted(data, old_gap_added);
        int gap_added_length = gap_added.length();
        if(gap_added_length == 0) {
            createErrorText(error);
            return;
        }

        cursor += DATA_LENGTH;
        statusBarText = gap_added.substr(GAP_SIZE);
        whiteSquare(image[TRACE]);
        redSquare(image[cursor]);

// Whitening to erase characters
        for(int i = TRACE; i <= IMAGE_COUNT; ++i) {
            p = image[i]->Canvas;
            p->Font->Name = "Arial";
            p->Font->Size = 10;

            if(i == cursor) {
                p->Font->Color = clRed;
            } else {
                p->Font->Color = clWhite;
            }

            SetBkMode(p->Handle, TRANSPARENT);

            if(i < OLD_GAP_ADDED_LENGTH) {
                c = old_gap_added.substr(i, 1);
                p->TextOut(text_x[c], TEXT_Y, c.c_str() );
            } else {
                p->TextOut(text_x[GAP], TEXT_Y, GAP.c_str() );
            }

        }

// Writing black characters
        for(int i = TRACE; i <= IMAGE_COUNT; ++i) {
            p = image[i]->Canvas;
            p->Font->Name = "Arial";
            p->Font->Size = 10;
            p->Font->Color = clBlack;
            SetBkMode(p->Handle, TRANSPARENT);

            if(i < gap_added_length) {
                c = gap_added.substr(i, 1);
                p->TextOut(text_x[c], TEXT_Y, c.c_str() );
            } else {
                p->TextOut(text_x[GAP], TEXT_Y, GAP.c_str() );
            }

        }

// undo data
        ud.cell_x = cursor;
        ud.previous_aim = aim;
        ud.previous_block_x = anchor;
        ud.previous_x = TRACE;
        ud.reverse_command = UNDO_INSERT;
        strcpy(ud.data, data.c_str() );
        ud.size_bytes = DATA_LENGTH;
        addUndoData();

// Checking if gap_added_length exceeds GAP_IMAGE_COUNT
        gap_added = GAP + statusBarText;
        gap_added_length = gap_added.length();
        if(gap_added_length > GAP_IMAGE_COUNT) {
            clipping();
        }

    }

}

void TForm1::jumpHandler(int jumpTo) {
    if(goMode) {
        using namespace std;

        const int TRACE = cursor;
        string gap_added = GAP + statusBarText;
        const int GAP_ADDED_LENGTH = gap_added.length();

        if(TRACE < 1) {
            homeHandler();
            return;
        } else if(TRACE >= GAP_ADDED_LENGTH) {
            endHandler();
            return;
        }

        cursor = jumpTo;
        if(cursor < 1) {
            cursor = 1;
        } else if(cursor >= GAP_ADDED_LENGTH) {
            cursor = GAP_ADDED_LENGTH - 1;
        }

        whiteSquare(image[TRACE]);
        redSquare(image[cursor]);

// Writing black characters
        blackCharacter(image[TRACE], gap_added, TRACE);
        blackCharacter(image[cursor], gap_added, cursor);

// undo data
        ud.reverse_command = UNDO_GO;
        ud.cell_x = cursor;
        ud.previous_aim = aim;
        ud.previous_x = TRACE;
        ud.previous_block_x = NO_BLOCK;
        memset(ud.data, 0, IMAGE_COUNT);
        ud.size_bytes = 0;
        addUndoData();
    }

}

void TForm1::leftHandler() {
    if(goMode) {
        using namespace std;

        const int TRACE = cursor;
        string gap_added = GAP + statusBarText;
        const int GAP_ADDED_LENGTH = gap_added.length();

        if(TRACE < 1) {
            homeHandler();
            return;
        } else if(TRACE >= GAP_ADDED_LENGTH) {
            endHandler();
            return;
        }

        cursor -= 1;
        if(cursor < 1) {
            cursor = 1;
        }

        whiteSquare(image[TRACE]);
        redSquare(image[cursor]);

// Writing black characters
        blackCharacter(image[TRACE], gap_added, TRACE);
        blackCharacter(image[cursor], gap_added, cursor);

// undo data
        ud.reverse_command = UNDO_GO;
        ud.cell_x = cursor;
        ud.previous_aim = aim;
        ud.previous_x = TRACE;
        ud.previous_block_x = NO_BLOCK;
        memset(ud.data, 0, IMAGE_COUNT);
        ud.size_bytes = 0;
        addUndoData();
    }

}

void TForm1::marginHandler() {
    if(goMode) {
        using namespace std;

        const bool ADD_MARGIN = true;
        const int DELETE_NOTHING = 0;
        string gap_added;
        string old_gap_added = GAP + statusBarText;

        if(editMode) {
            bs->addMargin(cursor);
        } else {
            editMode = true;
            bs = new BufferSplit(old_gap_added, cursor, DELETE_NOTHING, ADD_MARGIN);
        }

        gap_added = bs->unified();
        statusBarText = gap_added.substr(GAP_SIZE);

// undo data
        ud.cell_x = cursor;
        ud.previous_aim = aim;
        ud.previous_block_x = anchor;
        ud.previous_x = cursor;
        ud.reverse_command = UNDO_MARGIN;
        strcpy(ud.data, GAP.c_str() );
        ud.size_bytes = GAP_SIZE;
        addUndoData();
    }

}

void TForm1::no_swell() {
    using namespace std;

    const int TRACE = anchor;
    string gap_added = GAP + statusBarText;

    whiteSquare(image[TRACE]);
    blackCharacter(image[TRACE], gap_added, TRACE);
}

void TForm1::previousSwell(int previous_anchor) {
    using namespace std;

    const int TRACE = cursor;
    string gap_added = GAP + statusBarText;

    anchor = previous_anchor;

    for(int i = anchor; i <= TRACE; ++i) {
        aquaSquare(image[i]);
        blackCharacter(image[i], gap_added, i);
    }

}

void TForm1::redSquare(TImage *image) {
    TCanvas *p = image->Canvas;
    p->Brush->Color = clRed;
    p->FillRect( Rect(0, 0, IMAGE_WIDTH, IMAGE_HEIGHT) );

    if(widePane) {
        AnsiString CURSOR(cursor);
        AnsiString WHOLE( statusBarText.length() );
        Label1->Caption = CURSOR + " / " + WHOLE;
    }

}

void TForm1::sittingStill() {
    if(goMode) {
        using namespace std;

        string gap_added = GAP + statusBarText;
        redSquare(image[cursor]);

// Writing black characters
        blackCharacter(image[cursor], gap_added, cursor);

// undo data
        ud.reverse_command = UNDO_GO;
        ud.cell_x = cursor;
        ud.previous_aim = aim;
        ud.previous_x = cursor;
        ud.previous_block_x = NO_BLOCK;
        memset(ud.data, 0, IMAGE_COUNT);
        ud.size_bytes = 0;
        addUndoData();
    }

}

void TForm1::repeat_inserting(std::string & f) {
    if(goMode) {
        using namespace std;

        string full_data = f;

// Inserting characters one by one
        const int DATA_LENGTH = 1;
        const int FULL_DATA_LENGTH = full_data.length();
        int gap_added_length, old_gap_added_length, trace;
        string c, data, gap_added, old_gap_added;
        TCanvas *p;

        for(int k = 0; k < FULL_DATA_LENGTH; ++k) {
            data = full_data.substr(k, 1);

            trace = cursor;
            old_gap_added = GAP + statusBarText;
            old_gap_added_length = old_gap_added.length();

            gap_added = inserted(data, old_gap_added);
            gap_added_length = gap_added.length();
            if(gap_added_length == 0) {
                createErrorText(error);
                return;
            }

            cursor += 1;
            statusBarText = gap_added.substr(GAP_SIZE);

            whiteSquare(image[trace]);
            redSquare(image[cursor]);

// Whitening to erase characters
            for(int i = trace; i <= IMAGE_COUNT; ++i) {
                p = image[i]->Canvas;
                p->Font->Name = "Arial";
                p->Font->Size = 10;

                if(i == cursor) {
                    p->Font->Color = clRed;
                } else {
                    p->Font->Color = clWhite;
                }

                SetBkMode(p->Handle, TRANSPARENT);

                if(i < old_gap_added_length) {
                    c = old_gap_added.substr(i, 1);
                    p->TextOut(text_x[c], TEXT_Y, c.c_str() );
                } else {
                    p->TextOut(text_x[GAP], TEXT_Y, GAP.c_str() );
                }

            }

// Writing black characters
            for(int i = trace; i <= IMAGE_COUNT; ++i) {
                p = image[i]->Canvas;
                p->Font->Name = "Arial";
                p->Font->Size = 10;
                p->Font->Color = clBlack;
                SetBkMode(p->Handle, TRANSPARENT);

                if(i < gap_added_length) {
                    c = gap_added.substr(i, 1);
                    p->TextOut(text_x[c], TEXT_Y, c.c_str() );
                } else {
                    p->TextOut(text_x[GAP], TEXT_Y, GAP.c_str() );
                }

            }

// undo data
            ud.cell_x = cursor;
            ud.previous_aim = aim;
            ud.previous_block_x = anchor;
            ud.previous_x = trace;
            ud.reverse_command = UNDO_INSERT;
            strcpy(ud.data, data.c_str() );
            ud.size_bytes = DATA_LENGTH;
            addUndoData();

            if(cursor == IMAGE_COUNT) {
// Temporarily adding margin
                const bool ADD_MARGIN = true;
                const int DATA_LENGTH = 0;
                const int DELETE_NOTHING = 0;

                cursor = GAP_IMAGE_COUNT; // cursor = IMAGE_COUNT + 1;

 // Software may abandon a piece of data if the cursor reaches the end of the status bar.
                gap_added = gap_added.substr(0, GAP_IMAGE_COUNT);
                delete bs;
                bs = new BufferSplit(gap_added, cursor, DELETE_NOTHING, ADD_MARGIN);

                bs->excise(cursor, DATA_LENGTH);
                bs->trimming(cursor);
                gap_added = bs->unified();

                statusBarText = gap_added.substr(GAP_SIZE);

                cursor = IMAGE_COUNT;
                return;
            }

        }

    }

}

void TForm1::rightHandler() {
    if(goMode) {
        using namespace std;

        const int TRACE = cursor;
        string gap_added = GAP + statusBarText;
        const int GAP_ADDED_LENGTH = gap_added.length();

        if(TRACE < 1) {
            homeHandler();
            return;
        } else if(TRACE >= GAP_ADDED_LENGTH) {
            endHandler();
            return;
        }

        cursor += 1;
        if(cursor >= GAP_ADDED_LENGTH) {
            cursor = GAP_ADDED_LENGTH - 1;
        }

        whiteSquare(image[TRACE]);
        redSquare(image[cursor]);

// Writing black characters
        blackCharacter(image[TRACE], gap_added, TRACE);
        blackCharacter(image[cursor], gap_added, cursor);

// undo data
        ud.reverse_command = UNDO_GO;
        ud.cell_x = cursor;
        ud.previous_aim = aim;
        ud.previous_x = TRACE;
        ud.previous_block_x = NO_BLOCK;
        memset(ud.data, 0, IMAGE_COUNT);
        ud.size_bytes = 0;
        addUndoData();
    }

}

void TForm1::severHandler() {
    if(goMode) {
        using namespace std;

        if(undoClosed) {
            openUndo();
        }

// clipboard
        char buffer[SMALL_BUFFER_SIZE];
        string gap_added;
        string old_gap_added = GAP + statusBarText;
        string data = old_gap_added.substr(cursor);
        strcpy(buffer, data.c_str() );
        Clipboard()->SetTextBuf(buffer);

// Temporarily adding margin
        const bool ADD_MARGIN = true;
        const int DATA_LENGTH = data.length();
        const int DELETE_NOTHING = 0;

        if(editMode) {
            bs->addMargin(cursor);
        } else {
            editMode = true;
            bs = new BufferSplit(old_gap_added, cursor, DELETE_NOTHING, ADD_MARGIN);
        }

// Deleting
        bs->excise(cursor, DATA_LENGTH);
        gap_added = bs->unified();

        statusBarText = gap_added.substr(GAP_SIZE);
        redSquare(image[cursor]);

// Erasing characters
        for(int i = cursor + 1; i <= IMAGE_COUNT; ++i) {
            whiteSquare(image[i]);
        }

// undo data
        ud.cell_x = cursor;
        ud.previous_aim = aim;
        ud.previous_block_x = anchor;
        ud.previous_x = cursor;
        ud.reverse_command = UNDO_SEVER;
        strcpy(ud.data, data.c_str() );
        ud.size_bytes = DATA_LENGTH;
        addUndoData();
    }

}

void TForm1::showHandler() {
    using namespace std;

    if(blockMode) {
        copied = clearBlock();
    }

    aim = GO;
    goMode = true;

// Starting the Go mode
    string gap_added = GAP + statusBarText;
    int gap_added_length = gap_added.length();

    if(gap_added_length > GAP_IMAGE_COUNT) {
        gap_added_length = GAP_IMAGE_COUNT;
        gap_added = gap_added.substr(0, GAP_IMAGE_COUNT);
        statusBarText = gap_added.substr(GAP_SIZE);
    }

    if(cursor < 1) {
        cursor = 1;
    } else if(cursor >= gap_added_length) {
        cursor = gap_added_length - 1;
    }

    redSquare(image[cursor]);
    blackCharacter(image[cursor], gap_added, cursor);
}

void TForm1::shrinkHandler() {
    if(blockMode) {
        using namespace std;

        string gap_added = GAP + statusBarText;
        const int GAP_ADDED_LENGTH = gap_added.length();

        if(cursor < anchor) {
            cursor = anchor;
        } else if(cursor >= GAP_ADDED_LENGTH) {
            cursor = GAP_ADDED_LENGTH - 1;
        }

        const int TRACE = cursor;
        cursor -= 1;
        if(cursor < anchor) {
            cursor = anchor;
        }

        whiteSquare(image[TRACE]);
        aquaSquare(image[cursor]);

// Writing black characters
        blackCharacter(image[TRACE], gap_added, TRACE);
        blackCharacter(image[cursor], gap_added, cursor);

// undo data
        ud.reverse_command = UNDO_SHRINK;
        ud.cell_x = cursor;
        ud.previous_aim = aim;
        ud.previous_x = TRACE;
        ud.previous_block_x = anchor;
        memset(ud.data, 0, IMAGE_COUNT);
        ud.size_bytes = 0;
        addUndoData();
    }

}

void TForm1::startBlock() {
    using namespace std;

    if(blockMode) {
        copied = clearBlock();
    }

    endEditing();
    hideHandler();
    blockMode = true;

// Starting the Block mode
    string gap_added = GAP + statusBarText;
    const int GAP_ADDED_LENGTH = gap_added.length();

    if(cursor < 1) {
        return;
    } else if(cursor >= GAP_ADDED_LENGTH) {
        return;
    }

    ud.previous_block_x = anchor;
    anchor = cursor;
    aquaSquare(image[cursor]);
    blackCharacter(image[cursor], gap_added, cursor);

// undo data
    ud.previous_aim = aim;
    aim = SWELL;

    ud.reverse_command = UNDO_START;
    ud.cell_x = cursor;
    ud.previous_x = cursor;
    memset(ud.data, 0, IMAGE_COUNT);
    ud.size_bytes = 0;
    addUndoData();
}

void TForm1::statusBarMessage(std::string & s) {
    statusBarText = s;
    cursorLine();
}

void TForm1::swellHandler() {
    if(blockMode) {
        using namespace std;

        string gap_added = GAP + statusBarText;
        const int GAP_ADDED_LENGTH = gap_added.length();

        if(cursor < anchor) {
            cursor = anchor;
        } else if(cursor >= GAP_ADDED_LENGTH) {
            cursor = GAP_ADDED_LENGTH - 1;
        }

        cursor += 1;
        if(cursor >= GAP_ADDED_LENGTH) {
            cursor = GAP_ADDED_LENGTH - 1;
        }

        aquaSquare(image[cursor]);
        blackCharacter(image[cursor], gap_added, cursor);

// undo data
        ud.reverse_command = UNDO_SWELL;
        ud.cell_x = cursor;
        ud.previous_aim = aim;
        ud.previous_x = cursor - 1;
        ud.previous_block_x = anchor;
        memset(ud.data, 0, IMAGE_COUNT);
        ud.size_bytes = 0;
        addUndoData();
    }

}

void TForm1::undo_insert() {
    using namespace std;

    string data(ud.data);
    const int DATA_LENGTH = ud.size_bytes;
    const int TRACE = cursor;
    string c, gap_added;
    string old_gap_added = GAP + statusBarText;
    int OLD_GAP_ADDED_LENGTH = old_gap_added.length();

    cursor = ud.previous_x;
    if(editMode) {
        bs->excise(cursor, DATA_LENGTH);
        gap_added = bs->unified();
    } else {
        error = "e6: if(editMode" + CLOSE_BRACKET;
        return;
    }

    int GAP_ADDED_LENGTH = gap_added.length();
    statusBarText = gap_added.substr(GAP_SIZE);
    whiteSquare(image[TRACE]);
    redSquare(image[cursor]);
    whitenBlacken(old_gap_added, gap_added, OLD_GAP_ADDED_LENGTH, GAP_ADDED_LENGTH);
}

void TForm1::undoBS() {
    using namespace std;

    string data(ud.data);
    const int TRACE = cursor;
    string old_gap_added = GAP + statusBarText;
    int OLD_GAP_ADDED_LENGTH = old_gap_added.length();

    string gap_added = inserted(data, old_gap_added);
    int GAP_ADDED_LENGTH = gap_added.length();
    if(GAP_ADDED_LENGTH == 0) {
        createErrorText(error);
        return;
    }

    statusBarText = gap_added.substr(GAP_SIZE);
    whiteSquare(image[TRACE]);
    blackCharacter(image[TRACE], gap_added, TRACE);

    cursor = ud.previous_x;
    redSquare(image[cursor]);
    whitenBlacken(old_gap_added, gap_added, OLD_GAP_ADDED_LENGTH, GAP_ADDED_LENGTH);
}

void TForm1::undoClear() {
    previousSwell(ud.previous_block_x);
    blockMode = true;
}

void TForm1::undoClip() {
    using namespace std;

    string data(ud.data);
    string old_gap_added = GAP + statusBarText;

    bs->addData(data);
    string gap_added = bs->unified();
    statusBarText = gap_added.substr(GAP_SIZE);

    cursor = ud.previous_x;
    whiteSquare(image[IMAGE_COUNT]);
    whiteSquare(image[cursor]);

// Writing black characters
    blackCharacter(image[IMAGE_COUNT], gap_added, IMAGE_COUNT);
    blackCharacter(image[cursor], gap_added, cursor);

    if(recordNumber > 1) {
// Automatically proceeding to the undo_insert task
        if( readUndoBin() ) {
            gap_added = GAP + statusBarText;
            whiteSquare(image[cursor]);
            blackCharacter(image[cursor], gap_added, cursor);
        }

    }

// Ending the Go mode
    endEditing();
    goMode = false;
}

void TForm1::undoCut() {
    using namespace std;

    string data(ud.data);
    const int DATA_LENGTH = ud.size_bytes;
    const int TRACE = cursor;
    string c;
    string old_gap_added = GAP + statusBarText;
    int OLD_GAP_ADDED_LENGTH = old_gap_added.length();
    TCanvas *p;

    string gap_added = inserted(data, old_gap_added);
    const int GAP_ADDED_LENGTH = gap_added.length();
    if(GAP_ADDED_LENGTH == 0) {
        createErrorText(error);
        return;
    }

    cursor += DATA_LENGTH;
    cursor -= 1;
    statusBarText = gap_added.substr(GAP_SIZE);

    whiteSquare(image[TRACE]);

// Whitening to erase characters
    for(int i = TRACE; i <= IMAGE_COUNT; ++i) {
        p = image[i]->Canvas;
        p->Font->Name = "Arial";
        p->Font->Size = 10;
        p->Font->Color = clWhite;

        SetBkMode(p->Handle, TRANSPARENT);

        if(i < OLD_GAP_ADDED_LENGTH) {
            c = old_gap_added.substr(i, 1);
            p->TextOut(text_x[c], TEXT_Y, c.c_str() );
        } else {
            p->TextOut(text_x[GAP], TEXT_Y, GAP.c_str() );
        }

    }

// Writing black characters
    for(int i = TRACE; i <= IMAGE_COUNT; ++i) {
        p = image[i]->Canvas;
        p->Font->Name = "Arial";
        p->Font->Size = 10;
        p->Font->Color = clBlack;
        SetBkMode(p->Handle, TRANSPARENT);

        if(i < GAP_ADDED_LENGTH) {
            c = gap_added.substr(i, 1);
            p->TextOut(text_x[c], TEXT_Y, c.c_str() );
        } else {
            p->TextOut(text_x[GAP], TEXT_Y, GAP.c_str() );
        }

    }

// Restarting the Block mode
    aim = ud.previous_aim;
    previousSwell(ud.previous_block_x);
    blockMode = true;
}

void TForm1::undoCutAddMargin() {
    using namespace std;

    string data(ud.data);
    const int DATA_LENGTH = ud.size_bytes;
    const int TRACE = cursor;
    string c, gap_added;
    string old_gap_added = GAP + statusBarText;
    int OLD_GAP_ADDED_LENGTH = old_gap_added.length();
    TCanvas *p;

    cursor += 1;
    if( bs->squeezing(data, cursor) ) {
        bs->trimming(cursor);
        gap_added = bs->unified();
    } else {
        error = "e2: bs->squeezing(data, cursor" + CLOSE_BRACKET;
        createErrorText(error);
        return;
    }

    cursor += DATA_LENGTH;
    cursor -= 1;
    statusBarText = gap_added.substr(GAP_SIZE);

    whiteSquare(image[TRACE]);

// Whitening to erase characters
    const int GAP_ADDED_LENGTH = gap_added.length();
    for(int i = TRACE; i <= IMAGE_COUNT; ++i) {
        p = image[i]->Canvas;
        p->Font->Name = "Arial";
        p->Font->Size = 10;
        p->Font->Color = clWhite;

        SetBkMode(p->Handle, TRANSPARENT);

        if(i < OLD_GAP_ADDED_LENGTH) {
            c = old_gap_added.substr(i, 1);
            p->TextOut(text_x[c], TEXT_Y, c.c_str() );
        } else {
            p->TextOut(text_x[GAP], TEXT_Y, GAP.c_str() );
        }

    }

// Writing black characters
    for(int i = TRACE; i <= IMAGE_COUNT; ++i) {
        p = image[i]->Canvas;
        p->Font->Name = "Arial";
        p->Font->Size = 10;
        p->Font->Color = clBlack;
        SetBkMode(p->Handle, TRANSPARENT);

        if(i < GAP_ADDED_LENGTH) {
            c = gap_added.substr(i, 1);
            p->TextOut(text_x[c], TEXT_Y, c.c_str() );
        } else {
            p->TextOut(text_x[GAP], TEXT_Y, GAP.c_str() );
        }

    }

// Restarting the Block mode
    aim = ud.previous_aim;
    previousSwell(ud.previous_block_x);
    blockMode = true;
}

void TForm1::undoDelete() {
    using namespace std;

    string data(ud.data);
    const int TRACE = cursor;
    string old_gap_added = GAP + statusBarText;
    int OLD_GAP_ADDED_LENGTH = old_gap_added.length();

    cursor = ud.previous_x;
    if(cursor > TRACE) {
        whiteSquare(image[TRACE]);
        blackCharacter(image[TRACE], old_gap_added, TRACE);
    }

    string gap_added = inserted(data, old_gap_added);
    int GAP_ADDED_LENGTH = gap_added.length();
    if(GAP_ADDED_LENGTH == 0) {
        createErrorText(error);
        return;
    }

    statusBarText = gap_added.substr(GAP_SIZE);

    redSquare(image[cursor]);
    whitenBlacken(old_gap_added, gap_added, OLD_GAP_ADDED_LENGTH, GAP_ADDED_LENGTH);
}

void TForm1::undoGo() {
    using namespace std;

    const int TRACE = cursor;
    string gap_added = GAP + statusBarText;

    cursor = ud.previous_x;

    whiteSquare(image[TRACE]);
    redSquare(image[cursor]);

// Writing black characters
    blackCharacter(image[TRACE], gap_added, TRACE);
    blackCharacter(image[cursor], gap_added, cursor);
}

void TForm1::undoMargin() {
    using namespace std;

    string gap_added;
    string old_gap_added = GAP + statusBarText;

    cursor = ud.previous_x;
    if(editMode) {
        bs->trimming(cursor);
        gap_added = bs->unified();
    } else {
        error = "e7: if(editMode" + CLOSE_BRACKET;
        return;
    }

    statusBarText = gap_added.substr(GAP_SIZE);
}

void TForm1::undoSever() {
    using namespace std;

    string data(ud.data);
    string old_gap_added = GAP + statusBarText;
    int OLD_GAP_ADDED_LENGTH = old_gap_added.length();
    const int OLD_GAP_ADDED_LENGTH_1 = OLD_GAP_ADDED_LENGTH - 1;

// Removing GAP from the end
    old_gap_added = old_gap_added.substr(0, OLD_GAP_ADDED_LENGTH_1);
    OLD_GAP_ADDED_LENGTH = old_gap_added.length();
    bs->trimming(ud.previous_x);

    cursor = bs->addData(data);
    string gap_added = bs->unified();
    statusBarText = gap_added.substr(GAP_SIZE);
    int GAP_ADDED_LENGTH = gap_added.length();

    redSquare(image[cursor]);
    whitenBlacken(old_gap_added, gap_added, OLD_GAP_ADDED_LENGTH, GAP_ADDED_LENGTH);
}

void TForm1::undoShrink() {
    using namespace std;

    string gap_added = GAP + statusBarText;

    cursor = ud.previous_x;
    aquaSquare(image[cursor]);
    blackCharacter(image[cursor], gap_added, cursor);
}

void TForm1::undoStart() {
    if(ud.previous_aim == SWELL) {
        previousSwell(ud.previous_block_x);
    } else {
        no_swell();
    }

    aim = ud.previous_aim;
    switch(aim) {
    case GO:
        blockMode = false;
        showHandler();
        break;
    case SWELL:
        blockMode = true;
        break;
    }

}

void TForm1::undoSwell() {
    using namespace std;

    string gap_added = GAP + statusBarText;

    const int TRACE = cursor;
    cursor = ud.previous_x;

    whiteSquare(image[TRACE]);
    aquaSquare(image[cursor]);

// Writing black characters
    blackCharacter(image[TRACE], gap_added, TRACE);
    blackCharacter(image[cursor], gap_added, cursor);
}

void TForm1::whiteCharacter(TImage *image, std::string gap_added, int cell_x) {
    using namespace std;

    TCanvas *p = image->Canvas;
    p->Font->Name = "Arial";
    p->Font->Size = 10;
    p->Font->Color = clWhite;
    SetBkMode(p->Handle, TRANSPARENT);

    string c = gap_added.substr(cell_x, 1);
    p->TextOut(text_x[c], TEXT_Y, c.c_str() );
}

void TForm1::whitenBlacken(std::string & old_gap_added, std::string & gap_added, int OLD_GAP_ADDED_LENGTH, int GAP_ADDED_LENGTH) {
    using namespace std;

    string c;
    TCanvas *p;

// Whitening to erase characters
    for(int i = cursor + 1; i <= IMAGE_COUNT; ++i) {
        p = image[i]->Canvas;
        p->Font->Name = "Arial";
        p->Font->Size = 10;
        p->Font->Color = clWhite;

        SetBkMode(p->Handle, TRANSPARENT);

        if(i < OLD_GAP_ADDED_LENGTH) {
            c = old_gap_added.substr(i, 1);
            p->TextOut(text_x[c], TEXT_Y, c.c_str() );
        } else {
            p->TextOut(text_x[GAP], TEXT_Y, GAP.c_str() );
        }

    }

// Writing black characters
    for(int i = cursor; i <= IMAGE_COUNT; ++i) {
        p = image[i]->Canvas;
        p->Font->Name = "Arial";
        p->Font->Size = 10;
        p->Font->Color = clBlack;
        SetBkMode(p->Handle, TRANSPARENT);

        if(i < GAP_ADDED_LENGTH) {
            c = gap_added.substr(i, 1);
            p->TextOut(text_x[c], TEXT_Y, c.c_str() );
        } else {
            p->TextOut(text_x[GAP], TEXT_Y, GAP.c_str() );
        }

    }

}

std::string TForm1::whiteHandler() {
    const std::string SOMETHING_WHITE("               ");

    if(blockMode) {
        copied = clearBlock();
    }

    if(goMode) {
        endEditing();
        hideHandler();
    }

// Before undo
    if(! undoClosed) {
        putUndo.close();
        undoClosed = true;
        openUndo();
    }

    for(int i = 1; i <= IMAGE_COUNT; ++i) {
        whiteSquare(image[i]);
    }

    statusBarText = SOMETHING_WHITE;
    cursor = 1;
    return SOMETHING_WHITE;
}

void TForm1::whiteSquare(TImage *image) {
    TCanvas *p = image->Canvas;
    p->Brush->Color = clWhite;
    p->FillRect( Rect(0, 0, IMAGE_WIDTH, IMAGE_HEIGHT) );
}

void TForm1::aquaSquare(TImage *image) {
    TCanvas *p = image->Canvas;
    p->Brush->Color = aquamarine ? clAqua : clYellow;
    p->FillRect( Rect(0, 0, IMAGE_WIDTH, IMAGE_HEIGHT) );
}

__fastcall TForm1::TForm1(TComponent* Owner)
    : TForm(Owner), CAPITAL_A("A"), CAPITAL_B("B"), CAPITAL_C("C"), CAPITAL_D("D"), CAPITAL_E("E"), CAPITAL_F("F"), CAPITAL_G("G"), CAPITAL_H("H"), CAPITAL_I("I"), CAPITAL_J("J"), CLOSE_BRACKET(" )"), EDITOR_MENU("editorMenu.bmp"), EMPTY(""), GAP(" "), GREEN_SQUARE("DSD31.bmp"), MINUS("-"), NEW_LINE("\x0D\x0A"), NUMBER_0("0"), NUMBER_1("1"), NUMBER_2("2"), NUMBER_3("3"), NUMBER_4("4"), NUMBER_5("5"), NUMBER_6("6"), NUMBER_7("7"), NUMBER_8("8"), NUMBER_9("9"), RED_SQUARE("DSD6.bmp"), WHITE_SQUARE("DSD4.bmp")
{
    using namespace std;

    aquamarine = true;
    KeyPreview = true;
    Height = MAIN_HEIGHT;
    widePane = false;
    Width = MAIN_WIDTH;
    Panel1->Height = PANEL_HEIGHT;
    Panel1->Left = Image101->Width;
    Panel1->Align = alBottom;
    Position = poScreenCenter;

// hot spots in editorMenu.bmp (Image101)
    PaintBox1->Left = 1; PaintBox1->Top = 21; PaintBox1->Width = 105; PaintBox1->Height = 22; 
    PaintBox2->Left = 106; PaintBox2->Top = 16; PaintBox2->Width = 107; PaintBox2->Height = 18; 
    PaintBox3->Left = 343; PaintBox3->Top = 0; PaintBox3->Width = 149; PaintBox3->Height = 18; 
    PaintBox4->Left = 215; PaintBox4->Top = 170; PaintBox4->Width = 126; PaintBox4->Height = 19; 

    Image1->Top = 0;  Image1->Left = 0;    Image1->Height = IMAGE_HEIGHT;  Image1->Width = IMAGE_WIDTH;  Image1->Visible = true;
    Image2->Top = 0;  Image2->Left = 10;   Image2->Height = IMAGE_HEIGHT;  Image2->Width = IMAGE_WIDTH;  Image2->Visible = true;
    Image3->Top = 0;  Image3->Left = 20;   Image3->Height = IMAGE_HEIGHT;  Image3->Width = IMAGE_WIDTH;  Image3->Visible = true;
    Image4->Top = 0;  Image4->Left = 30;   Image4->Height = IMAGE_HEIGHT;  Image4->Width = IMAGE_WIDTH;  Image4->Visible = true;
    Image5->Top = 0;  Image5->Left = 40;   Image5->Height = IMAGE_HEIGHT;  Image5->Width = IMAGE_WIDTH;  Image5->Visible = true;
    Image6->Top = 0;  Image6->Left = 50;   Image6->Height = IMAGE_HEIGHT;  Image6->Width = IMAGE_WIDTH;  Image6->Visible = true;
    Image7->Top = 0;  Image7->Left = 60;   Image7->Height = IMAGE_HEIGHT;  Image7->Width = IMAGE_WIDTH;  Image7->Visible = true;
    Image8->Top = 0;  Image8->Left = 70;   Image8->Height = IMAGE_HEIGHT;  Image8->Width = IMAGE_WIDTH;  Image8->Visible = true;
    Image9->Top = 0;  Image9->Left = 80;   Image9->Height = IMAGE_HEIGHT;  Image9->Width = IMAGE_WIDTH;  Image9->Visible = true;
    Image10->Top = 0; Image10->Left = 90;  Image10->Height = IMAGE_HEIGHT; Image10->Width = IMAGE_WIDTH; Image10->Visible = true;
    Image11->Top = 0; Image11->Left = 100; Image11->Height = IMAGE_HEIGHT; Image11->Width = IMAGE_WIDTH; Image11->Visible = true;
    Image12->Top = 0; Image12->Left = 110; Image12->Height = IMAGE_HEIGHT; Image12->Width = IMAGE_WIDTH; Image12->Visible = true;
    Image13->Top = 0; Image13->Left = 120; Image13->Height = IMAGE_HEIGHT; Image13->Width = IMAGE_WIDTH; Image13->Visible = true;
    Image14->Top = 0; Image14->Left = 130; Image14->Height = IMAGE_HEIGHT; Image14->Width = IMAGE_WIDTH; Image14->Visible = true;
    Image15->Top = 0; Image15->Left = 140; Image15->Height = IMAGE_HEIGHT; Image15->Width = IMAGE_WIDTH; Image15->Visible = true;
    Image16->Top = 0; Image16->Left = 150; Image16->Height = IMAGE_HEIGHT; Image16->Width = IMAGE_WIDTH; Image16->Visible = true;
    Image17->Top = 0; Image17->Left = 160; Image17->Height = IMAGE_HEIGHT; Image17->Width = IMAGE_WIDTH; Image17->Visible = true;
    Image18->Top = 0; Image18->Left = 170; Image18->Height = IMAGE_HEIGHT; Image18->Width = IMAGE_WIDTH; Image18->Visible = true;
    Image19->Top = 0; Image19->Left = 180; Image19->Height = IMAGE_HEIGHT; Image19->Width = IMAGE_WIDTH; Image19->Visible = true;
    Image20->Top = 0; Image20->Left = 190; Image20->Height = IMAGE_HEIGHT; Image20->Width = IMAGE_WIDTH; Image20->Visible = true;
    Image21->Top = 0; Image21->Left = 200; Image21->Height = IMAGE_HEIGHT; Image21->Width = IMAGE_WIDTH; Image21->Visible = true;
    Image22->Top = 0; Image22->Left = 210; Image22->Height = IMAGE_HEIGHT; Image22->Width = IMAGE_WIDTH; Image22->Visible = true;
    Image23->Top = 0; Image23->Left = 220; Image23->Height = IMAGE_HEIGHT; Image23->Width = IMAGE_WIDTH; Image23->Visible = true;
    Image24->Top = 0; Image24->Left = 230; Image24->Height = IMAGE_HEIGHT; Image24->Width = IMAGE_WIDTH; Image24->Visible = true;
    Image25->Top = 0; Image25->Left = 240; Image25->Height = IMAGE_HEIGHT; Image25->Width = IMAGE_WIDTH; Image25->Visible = true;
    Image26->Top = 0; Image26->Left = 250; Image26->Height = IMAGE_HEIGHT; Image26->Width = IMAGE_WIDTH; Image26->Visible = true;
    Image27->Top = 0; Image27->Left = 260; Image27->Height = IMAGE_HEIGHT; Image27->Width = IMAGE_WIDTH; Image27->Visible = true;
    Image28->Top = 0; Image28->Left = 270; Image28->Height = IMAGE_HEIGHT; Image28->Width = IMAGE_WIDTH; Image28->Visible = true;
    Image29->Top = 0; Image29->Left = 280; Image29->Height = IMAGE_HEIGHT; Image29->Width = IMAGE_WIDTH; Image29->Visible = true;
    Image30->Top = 0; Image30->Left = 290; Image30->Height = IMAGE_HEIGHT; Image30->Width = IMAGE_WIDTH; Image30->Visible = true;
    Image31->Top = 0; Image31->Left = 300; Image31->Height = IMAGE_HEIGHT; Image31->Width = IMAGE_WIDTH; Image31->Visible = true;
    Image32->Top = 0; Image32->Left = 310; Image32->Height = IMAGE_HEIGHT; Image32->Width = IMAGE_WIDTH; Image32->Visible = true;
    Image33->Top = 0; Image33->Left = 320; Image33->Height = IMAGE_HEIGHT; Image33->Width = IMAGE_WIDTH; Image33->Visible = true;
    Image34->Top = 0; Image34->Left = 330; Image34->Height = IMAGE_HEIGHT; Image34->Width = IMAGE_WIDTH; Image34->Visible = true;
    Image35->Top = 0; Image35->Left = 340; Image35->Height = IMAGE_HEIGHT; Image35->Width = IMAGE_WIDTH; Image35->Visible = true;
    Image36->Top = 0; Image36->Left = 350; Image36->Height = IMAGE_HEIGHT; Image36->Width = IMAGE_WIDTH; Image36->Visible = true;
    Image37->Top = 0; Image37->Left = 360; Image37->Height = IMAGE_HEIGHT; Image37->Width = IMAGE_WIDTH; Image37->Visible = true;
    Image38->Top = 0; Image38->Left = 370; Image38->Height = IMAGE_HEIGHT; Image38->Width = IMAGE_WIDTH; Image38->Visible = true;
    Image39->Top = 0; Image39->Left = 380; Image39->Height = IMAGE_HEIGHT; Image39->Width = IMAGE_WIDTH; Image39->Visible = true;
    Image40->Top = 0; Image40->Left = 390; Image40->Height = IMAGE_HEIGHT; Image40->Width = IMAGE_WIDTH; Image40->Visible = true;
    Image41->Top = 0; Image41->Left = 400; Image41->Height = IMAGE_HEIGHT; Image41->Width = IMAGE_WIDTH; Image41->Visible = true;
    Image42->Top = 0; Image42->Left = 410; Image42->Height = IMAGE_HEIGHT; Image42->Width = IMAGE_WIDTH; Image42->Visible = true;
    Image43->Top = 0; Image43->Left = 420; Image43->Height = IMAGE_HEIGHT; Image43->Width = IMAGE_WIDTH; Image43->Visible = true;
    Image44->Top = 0; Image44->Left = 430; Image44->Height = IMAGE_HEIGHT; Image44->Width = IMAGE_WIDTH; Image44->Visible = true;
    Image45->Top = 0; Image45->Left = 440; Image45->Height = IMAGE_HEIGHT; Image45->Width = IMAGE_WIDTH; Image45->Visible = true;
    Image46->Top = 0; Image46->Left = 450; Image46->Height = IMAGE_HEIGHT; Image46->Width = IMAGE_WIDTH; Image46->Visible = true;
    Image47->Top = 0; Image47->Left = 460; Image47->Height = IMAGE_HEIGHT; Image47->Width = IMAGE_WIDTH; Image47->Visible = true;
    Image48->Top = 0; Image48->Left = 470; Image48->Height = IMAGE_HEIGHT; Image48->Width = IMAGE_WIDTH; Image48->Visible = true;
    Image49->Top = 0; Image49->Left = 480; Image49->Height = IMAGE_HEIGHT; Image49->Width = IMAGE_WIDTH; Image49->Visible = true;
    Image50->Top = 0; Image50->Left = 490; Image50->Height = IMAGE_HEIGHT; Image50->Width = IMAGE_WIDTH; Image50->Visible = true;
    Image51->Top = 0; Image51->Left = 500; Image51->Height = IMAGE_HEIGHT; Image51->Width = IMAGE_WIDTH; Image51->Visible = true;
    Image52->Top = 0; Image52->Left = 510; Image52->Height = IMAGE_HEIGHT; Image52->Width = IMAGE_WIDTH; Image52->Visible = true;
    Image53->Top = 0; Image53->Left = 520; Image53->Height = IMAGE_HEIGHT; Image53->Width = IMAGE_WIDTH; Image53->Visible = true;
    Image54->Top = 0; Image54->Left = 530; Image54->Height = IMAGE_HEIGHT; Image54->Width = IMAGE_WIDTH; Image54->Visible = true;
    Image55->Top = 0; Image55->Left = 540; Image55->Height = IMAGE_HEIGHT; Image55->Width = IMAGE_WIDTH; Image55->Visible = true;
    Image56->Top = 0; Image56->Left = 550; Image56->Height = IMAGE_HEIGHT; Image56->Width = IMAGE_WIDTH; Image56->Visible = true;
    Image57->Top = 0; Image57->Left = 560; Image57->Height = IMAGE_HEIGHT; Image57->Width = IMAGE_WIDTH; Image57->Visible = true;
    Image58->Top = 0; Image58->Left = 570; Image58->Height = IMAGE_HEIGHT; Image58->Width = IMAGE_WIDTH; Image58->Visible = true;
    Image59->Top = 0; Image59->Left = 580; Image59->Height = IMAGE_HEIGHT; Image59->Width = IMAGE_WIDTH; Image59->Visible = true;
    Image60->Top = 0; Image60->Left = 590; Image60->Height = IMAGE_HEIGHT; Image60->Width = IMAGE_WIDTH; Image60->Visible = true;
    Image61->Top = 0; Image61->Left = 600; Image61->Height = IMAGE_HEIGHT; Image61->Width = IMAGE_WIDTH; Image61->Visible = true;
    Image62->Top = 0; Image62->Left = 610; Image62->Height = IMAGE_HEIGHT; Image62->Width = IMAGE_WIDTH; Image62->Visible = true;
    Image63->Top = 0; Image63->Left = 620; Image63->Height = IMAGE_HEIGHT; Image63->Width = IMAGE_WIDTH; Image63->Visible = true;
    Image64->Top = 0; Image64->Left = 630; Image64->Height = IMAGE_HEIGHT; Image64->Width = IMAGE_WIDTH; Image64->Visible = true;
    Image65->Top = 0; Image65->Left = 640; Image65->Height = IMAGE_HEIGHT; Image65->Width = IMAGE_WIDTH; Image65->Visible = true;
    Image66->Top = 0; Image66->Left = 650; Image66->Height = IMAGE_HEIGHT; Image66->Width = IMAGE_WIDTH; Image66->Visible = true;
    Image67->Top = 0; Image67->Left = 660; Image67->Height = IMAGE_HEIGHT; Image67->Width = IMAGE_WIDTH; Image67->Visible = true;
    Image68->Top = 0; Image68->Left = 670; Image68->Height = IMAGE_HEIGHT; Image68->Width = IMAGE_WIDTH; Image68->Visible = true;
    Image69->Top = 0; Image69->Left = 680; Image69->Height = IMAGE_HEIGHT; Image69->Width = IMAGE_WIDTH; Image69->Visible = true;
    Image70->Top = 0; Image70->Left = 690; Image70->Height = IMAGE_HEIGHT; Image70->Width = IMAGE_WIDTH; Image70->Visible = true;
    Image71->Top = 0; Image71->Left = 700; Image71->Height = IMAGE_HEIGHT; Image71->Width = IMAGE_WIDTH; Image71->Visible = true;
    Image72->Top = 0; Image72->Left = 710; Image72->Height = IMAGE_HEIGHT; Image72->Width = IMAGE_WIDTH; Image72->Visible = true;
    Image73->Top = 0; Image73->Left = 720; Image73->Height = IMAGE_HEIGHT; Image73->Width = IMAGE_WIDTH; Image73->Visible = true;
    Image74->Top = 0; Image74->Left = 730; Image74->Height = IMAGE_HEIGHT; Image74->Width = IMAGE_WIDTH; Image74->Visible = true;
    Image75->Top = 0; Image75->Left = 740; Image75->Height = IMAGE_HEIGHT; Image75->Width = IMAGE_WIDTH; Image75->Visible = true;
    Image76->Top = 0; Image76->Left = 750; Image76->Height = IMAGE_HEIGHT; Image76->Width = IMAGE_WIDTH; Image76->Visible = true;
    Image77->Top = 0; Image77->Left = 760; Image77->Height = IMAGE_HEIGHT; Image77->Width = IMAGE_WIDTH; Image77->Visible = true;
    Image78->Top = 0; Image78->Left = 770; Image78->Height = IMAGE_HEIGHT; Image78->Width = IMAGE_WIDTH; Image78->Visible = true;
    Image79->Top = 0; Image79->Left = 780; Image79->Height = IMAGE_HEIGHT; Image79->Width = IMAGE_WIDTH; Image79->Visible = true;
    Image80->Top = 0; Image80->Left = 790; Image80->Height = IMAGE_HEIGHT; Image80->Width = IMAGE_WIDTH; Image80->Visible = true;
    Image81->Top = 0; Image81->Left = 800; Image81->Height = IMAGE_HEIGHT; Image81->Width = IMAGE_WIDTH; Image81->Visible = true;
    Image82->Top = 0; Image82->Left = 810; Image82->Height = IMAGE_HEIGHT; Image82->Width = IMAGE_WIDTH; Image82->Visible = true;
    Image83->Top = 0; Image83->Left = 820; Image83->Height = IMAGE_HEIGHT; Image83->Width = IMAGE_WIDTH; Image83->Visible = true;
    Image84->Top = 0; Image84->Left = 830; Image84->Height = IMAGE_HEIGHT; Image84->Width = IMAGE_WIDTH; Image84->Visible = true;
    Image85->Top = 0; Image85->Left = 840; Image85->Height = IMAGE_HEIGHT; Image85->Width = IMAGE_WIDTH; Image85->Visible = true;
    Image86->Top = 0; Image86->Left = 850; Image86->Height = IMAGE_HEIGHT; Image86->Width = IMAGE_WIDTH; Image86->Visible = true;
    Image87->Top = 0; Image87->Left = 860; Image87->Height = IMAGE_HEIGHT; Image87->Width = IMAGE_WIDTH; Image87->Visible = true;
    Image88->Top = 0; Image88->Left = 870; Image88->Height = IMAGE_HEIGHT; Image88->Width = IMAGE_WIDTH; Image88->Visible = true;
    Image89->Top = 0; Image89->Left = 880; Image89->Height = IMAGE_HEIGHT; Image89->Width = IMAGE_WIDTH; Image89->Visible = true;
    Image90->Top = 0; Image90->Left = 890; Image90->Height = IMAGE_HEIGHT; Image90->Width = IMAGE_WIDTH; Image90->Visible = true;
    Image91->Top = 0; Image91->Left = 900; Image91->Height = IMAGE_HEIGHT; Image91->Width = IMAGE_WIDTH; Image91->Visible = true;
    Image92->Top = 0; Image92->Left = 910; Image92->Height = IMAGE_HEIGHT; Image92->Width = IMAGE_WIDTH; Image92->Visible = true;
    Image93->Top = 0; Image93->Left = 920; Image93->Height = IMAGE_HEIGHT; Image93->Width = IMAGE_WIDTH; Image93->Visible = true;
    Image94->Top = 0; Image94->Left = 930; Image94->Height = IMAGE_HEIGHT; Image94->Width = IMAGE_WIDTH; Image94->Visible = true;
    Image95->Top = 0; Image95->Left = 940; Image95->Height = IMAGE_HEIGHT; Image95->Width = IMAGE_WIDTH; Image95->Visible = true;
    Image96->Top = 0; Image96->Left = 950; Image96->Height = IMAGE_HEIGHT; Image96->Width = IMAGE_WIDTH; Image96->Visible = true;
    Image97->Top = 0; Image97->Left = 960; Image97->Height = IMAGE_HEIGHT; Image97->Width = IMAGE_WIDTH; Image97->Visible = true;
    Image98->Top = 0; Image98->Left = 970; Image98->Height = IMAGE_HEIGHT; Image98->Width = IMAGE_WIDTH; Image98->Visible = true;
    Image99->Top = 0; Image99->Left = 980; Image99->Height = IMAGE_HEIGHT; Image99->Width = IMAGE_WIDTH; Image99->Visible = true;
    Image100->Top = 0; Image100->Left = 990; Image100->Height = IMAGE_HEIGHT; Image100->Width = IMAGE_WIDTH; Image100->Visible = true;

    image[0] = NULL;
    image[1] = Image1; image[2] = Image2; image[3] = Image3; image[4] = Image4; image[5] = Image5; image[6] = Image6; image[7] = Image7; image[8] = Image8; image[9] = Image9; image[10] = Image10; 
    image[11] = Image11; image[12] = Image12; image[13] = Image13; image[14] = Image14; image[15] = Image15; image[16] = Image16; image[17] = Image17; image[18] = Image18; image[19] = Image19; image[20] = Image20; 
    image[21] = Image21; image[22] = Image22; image[23] = Image23; image[24] = Image24; image[25] = Image25; image[26] = Image26; image[27] = Image27; image[28] = Image28; image[29] = Image29; image[30] = Image30; 
    image[31] = Image31; image[32] = Image32; image[33] = Image33; image[34] = Image34; image[35] = Image35; image[36] = Image36; image[37] = Image37; image[38] = Image38; image[39] = Image39; image[40] = Image40; 
    image[41] = Image41; image[42] = Image42; image[43] = Image43; image[44] = Image44; image[45] = Image45; image[46] = Image46; image[47] = Image47; image[48] = Image48; image[49] = Image49; image[50] = Image50; 
    image[51] = Image51; image[52] = Image52; image[53] = Image53; image[54] = Image54; image[55] = Image55; image[56] = Image56; image[57] = Image57; image[58] = Image58; image[59] = Image59; image[60] = Image60; 
    image[61] = Image61; image[62] = Image62; image[63] = Image63; image[64] = Image64; image[65] = Image65; image[66] = Image66; image[67] = Image67; image[68] = Image68; image[69] = Image69; image[70] = Image70; 
    image[71] = Image71; image[72] = Image72; image[73] = Image73; image[74] = Image74; image[75] = Image75; image[76] = Image76; image[77] = Image77; image[78] = Image78; image[79] = Image79; image[80] = Image80; 
    image[81] = Image81; image[82] = Image82; image[83] = Image83; image[84] = Image84; image[85] = Image85; image[86] = Image86; image[87] = Image87; image[88] = Image88; image[89] = Image89; image[90] = Image90; 
    image[91] = Image91; image[92] = Image92; image[93] = Image93; image[94] = Image94; image[95] = Image95; image[96] = Image96; image[97] = Image97; image[98] = Image98; image[99] = Image99; image[100] = Image100; 

    Image101->Picture->LoadFromFile(EDITOR_MENU);

// Software uses text_x[c] and TEXT_Y to adjust offset from the corner of image[n].
    const string ASCII("!\"#$%&'()*+,./:;<=>?@KLMNOPQRSTUVWXYZ[\\]^_`abcdefghijklmnopqrstuvwxyz{|}~");
    const int ASCII_LENGTH = ASCII.length();
    string c;
    for(int i = 0; i < ASCII_LENGTH; ++i) {
        c = ASCII.substr(i, 1);
        text_x[c] = 2;
    }

    text_x[CAPITAL_A] = 1;
    text_x[CAPITAL_B] = 1;
    text_x[CAPITAL_C] = 1;
    text_x[CAPITAL_D] = 1;
    text_x[CAPITAL_E] = 1;
    text_x[CAPITAL_F] = 1;
    text_x[CAPITAL_G] = 1;
    text_x[CAPITAL_H] = 1;
    text_x[CAPITAL_I] = 4;
    text_x[CAPITAL_J] = 1;
    text_x[GAP] = 2;
    text_x[MINUS] = 3;
    text_x[NUMBER_0] = 2;
    text_x[NUMBER_1] = 3;
    text_x[NUMBER_2] = 2;
    text_x[NUMBER_3] = 2;
    text_x[NUMBER_4] = 2;
    text_x[NUMBER_5] = 2;
    text_x[NUMBER_6] = 2;
    text_x[NUMBER_7] = 2;
    text_x[NUMBER_8] = 2;
    text_x[NUMBER_9] = 2;

    cellNumber = ""; // AnsiString
    fullPathName = em->fullPathName;
    line_1 = em->line_1;
    versionNumber = em->versionNumber;

    aim = NO_AIM;
    anchor = 1;
    blockMode = false;
    bs = NULL; // bs = new BufferSplit(data, s, cursor);
    copied = EMPTY;
    cursor = 1;
    editMode = false;
    error = EMPTY;
    goMode = false;

// Before undo
    undoClosed = true;
    openUndo();

    statusBarText = line_1;
    cursorLine();
    unhandled = modified_file(MANAGE_TEXT);
    Timer1->Interval = NON_SAMPLING_PERIOD;
}

void __fastcall TForm1::Exit1Click(TObject *Sender)
{
    Close();
}

void __fastcall TForm1::Version1Click(TObject *Sender)
{
    executeProgram(ZOKO_PDF, "");
}

void __fastcall TForm1::Version2Click(TObject *Sender)
{
    AnsiString VERSION_CAPTION("Editor");;
    if(versionCaption) {
        versionCaption = false;
    } else {
        AnsiString VERSION_NUMBER(versionNumber);
        versionCaption = true;
        VERSION_CAPTION += " Version ";
        VERSION_CAPTION += VERSION_NUMBER;
    }

    Caption = VERSION_CAPTION;
}

void __fastcall TForm1::Wide1Click(TObject *Sender)
{
    if(widePane) {
        widePane = false;
        Left = MAIN_X;
        Width = MAIN_WIDTH;
    } else {
        AnsiString CURSOR(cursor);
        AnsiString WHOLE( statusBarText.length() );
        Label1->Caption = CURSOR + " / " + WHOLE;
        widePane = true;
        Left = 0;
        Width = WIDE_PANE;
    }

}

void __fastcall TForm1::Show1Click(TObject *Sender)
{
    showHandler();
    sittingStill();
}

void __fastcall TForm1::Left1Click(TObject *Sender)
{
    leftHandler();
}

void __fastcall TForm1::Right1Click(TObject *Sender)
{
    rightHandler();
}

void __fastcall TForm1::Undo1Click(TObject *Sender)
{
    undoThat();
}

void __fastcall TForm1::FormKeyDown(TObject *Sender, WORD &Key,
      TShiftState Shift)
{
	switch(Key) {
	case '0':
        insertHandler(NUMBER_0);
        break;
	case '1':
        insertHandler(NUMBER_1);
        break;
	case '2':
        insertHandler(NUMBER_2);
        break;
	case '3':
        insertHandler(NUMBER_3);
        break;
	case '4':
        insertHandler(NUMBER_4);
        break;
	case '5':
        insertHandler(NUMBER_5);
        break;
	case '6':
        insertHandler(NUMBER_6);
        break;
	case '7':
        insertHandler(NUMBER_7);
        break;
	case '8':
        insertHandler(NUMBER_8);
        break;
	case '9':
        insertHandler(NUMBER_9);
        break;
	case 'A':
        insertHandler(CAPITAL_A);
        break;
	case 'B':
        insertHandler(CAPITAL_B);
        break;
	case 'C':
        insertHandler(CAPITAL_C);
        break;
	case 'D':
        insertHandler(CAPITAL_D);
        break;
	case 'E':
        insertHandler(CAPITAL_E);
        break;
	case 'F':
        insertHandler(CAPITAL_F);
        break;
	case 'G':
        insertHandler(CAPITAL_G);
        break;
	case 'H':
        insertHandler(CAPITAL_H);
        break;
	case 'I':
        insertHandler(CAPITAL_I);
        break;
	case 'J':
        insertHandler(CAPITAL_J);
        break;
	case 'M':
        insertHandler(MINUS);
        break;
	case VK_BACK:// Back Space (BS)
        BShandler();
        break;
    case VK_DELETE:
        deleteHandler();
        break;
    case VK_END:
        endHandler();
        break;
    case VK_HOME:
        homeHandler();
        break;
    case VK_INSERT:
        insertHandler(copied);
        break;
    case VK_LEFT:
        leftHandler();
        break;
    case VK_NEXT:
        shrinkHandler();
        break;
    case VK_PRIOR:
        swellHandler();
        break;
    case VK_RIGHT:
        rightHandler();
        break;
	case VK_SPACE:
        insertHandler(GAP);
        break;
	case VK_SUBTRACT:// -
        insertHandler(MINUS);
        break;
    default:
        return;
    }

}

void __fastcall TForm1::Copy1Click(TObject *Sender)
{
    copyHandler();
}

void __fastcall TForm1::Hide1Click(TObject *Sender)
{
    hideHandler();
}

void __fastcall TForm1::Home1Click(TObject *Sender)
{
    homeHandler();
}

void __fastcall TForm1::End1Click(TObject *Sender)
{
    endHandler();
}

void __fastcall TForm1::Delete1Click(TObject *Sender)
{
    deleteHandler();
}

void __fastcall TForm1::BS1Click(TObject *Sender)
{
    BShandler();
}

void __fastcall TForm1::Minus1Click(TObject *Sender)
{
    insertHandler(MINUS);
}

void __fastcall TForm1::Space1Click(TObject *Sender)
{
    insertHandler(GAP);
}

void __fastcall TForm1::A1Click(TObject *Sender)
{
    insertHandler(CAPITAL_A);
}

void __fastcall TForm1::B1Click(TObject *Sender)
{
    insertHandler(CAPITAL_B);
}

void __fastcall TForm1::C1Click(TObject *Sender)
{
    insertHandler(CAPITAL_C);
}

void __fastcall TForm1::D1Click(TObject *Sender)
{
    insertHandler(CAPITAL_D);
}

void __fastcall TForm1::E1Click(TObject *Sender)
{
    insertHandler(CAPITAL_E);
}

void __fastcall TForm1::F1Click(TObject *Sender)
{
    insertHandler(CAPITAL_F);
}

void __fastcall TForm1::G1Click(TObject *Sender)
{
    insertHandler(CAPITAL_G);
}

void __fastcall TForm1::H1Click(TObject *Sender)
{
    insertHandler(CAPITAL_H);
}

void __fastcall TForm1::I1Click(TObject *Sender)
{
    insertHandler(CAPITAL_I);
}

void __fastcall TForm1::J1Click(TObject *Sender)
{
    insertHandler(CAPITAL_J);
}

void __fastcall TForm1::FormDestroy(TObject *Sender)
{
    endEditing();
    createRemoveErrorText();
}

void __fastcall TForm1::Shrink1Click(TObject *Sender)
{
    shrinkHandler();
}

void __fastcall TForm1::Swell1Click(TObject *Sender)
{
    swellHandler();
}

void __fastcall TForm1::Cut1Click(TObject *Sender)
{
    copied = cutHandler();
}

void __fastcall TForm1::Insert1Click(TObject *Sender)
{
    insertHandler(copied);
}

void __fastcall TForm1::Image1Click(TObject *Sender)
{
    showHandler();
    homeHandler();
}

void __fastcall TForm1::Image100Click(TObject *Sender)
{
    showHandler();
    endHandler();
}

void __fastcall TForm1::Clipboard1Click(TObject *Sender)
{
    acquireDataFromClipBoard();
}

void __fastcall TForm1::Sever1Click(TObject *Sender)
{
    severHandler();
}

void __fastcall TForm1::Margin1Click(TObject *Sender)
{
    marginHandler();
}

void __fastcall TForm1::White1Click(TObject *Sender)
{
    whiteHandler();
}

void __fastcall TForm1::Save1Click(TObject *Sender)
{
    saveLine1();
}

void __fastcall TForm1::Timer1Timer(TObject *Sender)
{
// Checking if MANAGE_TEXT has been modified
    int m = modified_file(MANAGE_TEXT);
    if(m != unhandled) {
        unhandled = m;
        readManageText();
    }

}

void __fastcall TForm1::Inquiry1Click(TObject *Sender)
{
    copied = clearBlock();
}

void __fastcall TForm1::Jump1Click(TObject *Sender)
{
    AnsiString CURSOR(cursor);
    AnsiString WHOLE( statusBarText.length() );
    AnsiString cursorWhole = CURSOR + " / " + WHOLE;
    Label1->Caption = cursorWhole;
    OKBottomDlg->info(cursorWhole);

    if( OKBottomDlg->ShowModal() == mrOk ) {
        cellNumber = em->cellNumber;
        const int CELL_NUMBER_LENGTH = cellNumber.Length();
        const int NON_NUMBERS = strcspn( cellNumber.c_str(), "0123456789");

        int from = NON_NUMBERS + 1;
        int count = CELL_NUMBER_LENGTH - NON_NUMBERS;
        AnsiString figures = cellNumber.SubString(from, count);

        if(! widePane) {
            widePane = true;
            Left = 0;
            Width = WIDE_PANE;
        }

        jumpTo = figures.ToInt();
        jumpHandler(jumpTo);
    }

}

void __fastcall TForm1::Aquamarine1Click(TObject *Sender)
{
    startBlock();
}

void __fastcall TForm1::Yellow1Click(TObject *Sender)
{
    aquamarine = !aquamarine;
}

void __fastcall TForm1::PaintBox1Click(TObject *Sender)
{
    Close();
}

void __fastcall TForm1::PaintBox2Click(TObject *Sender)
{
    hideHandler();
}

void __fastcall TForm1::PaintBox3Click(TObject *Sender)
{
    cutHandler();
}

void __fastcall TForm1::PaintBox4Click(TObject *Sender)
{
    whiteHandler();
}

