#include "FormDateLine.h"

bool FormDateLine::altering(std::string & fileName) {
    using namespace std;

    ifstream in( fileName.c_str() );
    if( in.fail() ) {
        error = "e27: ifstream in( " + fileName + CLOSE_BRACKET;
        return false;
    }

    char c[BUFFER_SIZE];
    string empty(EMPTY);
    if( in.getline(c, BUFFER_SIZE) ) {
        string line(c);
        if(firstLine) {
            add(line);
            listing();
            clear(empty);
        }

    } else {
        error = "e28(Line-1): in.getline(c, BUFFER_SIZE" + CLOSE_BRACKET;
        return false;
    }

    string *p = new string("");
    Characterizable *lineType = new DateFollowed(*p);
    int q, r;
    string s = EMPTY;
    stringstream *os = new stringstream();

    if(whoText) {
        saveName = WHO_TEXT;
    } else {
        TimeName wt(WHO, TEXT_FILE);
        saveName = wt.temporaryName(serialNumber);
    }

// ring buffer
    produceRingBuffer(RING_COUNT);

    while( in.getline(c, BUFFER_SIZE) ) {
        delete p;
        p = new string(c);
        ring.pop_front();
        ring.push_back(*p);

        q = p->find(ID_COLON);
        if(q != string::npos) {
            delete os;
            os = new stringstream();
            RING::iterator i = ring.begin();
            *os << *i;
            ++i;
            for( ; i != ring.end(); ++i) {
                *os << *i << GAP;
            }

            s = os->str();
            delete lineType;
            lineType = new DateFollowed(s);
            r = lineType->dateLine();
            if(r > 0) {
                listing();

                s = lineType->interposed();
                clear(s);
                blankRingBuffer();
                continue;
            } else {
                RING::iterator i = ring.begin();
                s = *i;
            }

        } else {
            RING::iterator i = ring.begin();
            s = *i;
        }

        add(s);
    }

    in.close();

    ring.pop_front();
    for(RING::iterator i = ring.begin(); i != ring.end(); ++i) {
        add(*i);
    }

    listing();
    openSaveClose(saveName);

    delete lineType;
    delete os;
    delete p;
    return true;
}

bool FormDateLine::openSaveClose(std::string & fileName) {
    using namespace std;

// Trimming
    string s = buffer->str();
    int size = s.length();
    string c;
    for(int i = 0; i < size; i += NEW_LINE_LENGTH) {
        c = s.substr(i, NEW_LINE_LENGTH);
        if(NEW_LINE == c) {
            continue;
        } else {
            s = s.substr(i);
            break;
        }

    }

// Writing
    ofstream saveFile( fileName.c_str() );
    if( saveFile.fail() ) {
        error = "e1: ofstream saveFile( " + fileName + CLOSE_BRACKET;
        return false;
    }

    saveFile << s;
    saveFile.close();
    return true;
}

int FormDateLine::produceRingBuffer(int size) {
    using namespace std;

    string BLANK(" ");
    ring.clear();
    for(int i = 0; i < size; i += 1) {
        ring.push_back(BLANK);
    }

    return ring.size();
}

void FormDateLine::blankRingBuffer() {
    using namespace std;     

    string BLANK(" ");
    ring.pop_front();
    ring.push_back(BLANK);
    ring.pop_front();
    ring.push_back(BLANK);
    ring.pop_front();
    ring.push_back(BLANK);
}
